"""
Purpose: Given a pull request, mirror the bug in the current form of the repository.

Usage: python -m swesmith.bug_gen.mirror.generate logs/prs/data/*-task-instances.jsonl
"""

import argparse
import json
import litellm
import logging
import os
import re
import shutil
import uuid
import traceback
import signal

from concurrent.futures import ProcessPoolExecutor, as_completed
from dotenv import load_dotenv
from litellm import completion, completion_cost
from multiprocessing import current_process
from swebench.harness.constants import KEY_INSTANCE_ID
from swesmith.bug_gen.utils import (
    apply_patches,
    get_patch,
)
from swesmith.bug_gen.mirror.prompts import (
    DEMO_PROMPT,
    RECOVERY_PROMPT,
    TASK_PROMPT,
)
from swesmith.constants import (
    LOG_DIR_BUG_GEN,
    KEY_PATCH,
    PREFIX_BUG,
    PREFIX_METADATA,
    INSTANCE_REF,
)
from swesmith.profiles import registry, RepoProfile
from tqdm.auto import tqdm
from unidiff import PatchSet

load_dotenv()
litellm.drop_params = True

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

logging.getLogger("LiteLLM").setLevel(logging.WARNING)
litellm.suppress_debug_info = True

MIRROR_PR = "pr_mirror"
KEY_COST = "cost"
KEY_PULL_NUM = "pull_number"
KEY_RECOVER_STATUS = "recover_status"
KEY_REWRITES = "rewrites"
KEY_SKIP_REASON = "skip_reason"
RECOVER_FAIL = "failed"
RECOVER_SKIPPED = "skipped"
RECOVER_SUCCESS = "success"


def get_metadata_file_name(pr_num):
    return f"{PREFIX_METADATA}__pr_{pr_num}.json"


worker_tempdirs = {}


def should_attempt_recovery(
    inst, repo, max_files=8, max_lines=500, max_file_lines=10000
):
    """
    Attempt if the following criteria are met:
    * Fewer than max_files files are changed
    * Fewer than max_lines lines are changed
    * No changed file is >max_file_lines lines
    """
    patch = PatchSet(inst[KEY_PATCH])
    num_py_edited = len([x for x in patch if x.path.endswith(".py")])
    if num_py_edited == 0:
        return False, "No Python files changed"
    if num_py_edited > max_files:
        return False, f"Too many files changed (>{max_files} files)"
    lines_changed = 0
    for file_diff in patch:
        if file_diff.is_binary_file:
            return False, "Contains binary file"
        file_path = os.path.join(repo, file_diff.path)
        if not os.path.exists(file_path):
            # Skip over edits to files that don't exist
            continue
        file_content = open(file_path).read()
        if len(file_content.splitlines()) > max_file_lines:
            return False, f"Changed file is too long (>{max_file_lines} lines)"
        lines_changed += file_diff.added + file_diff.removed
    if lines_changed == 0:
        return False, "No lines changed (no changed file exists)"
    if lines_changed > max_lines:
        return False, f"Too many lines changed (>{max_lines})"
    return True, None


def recover_sweb_inst(inst, repo, model, api_key=None, log_path=None):
    """
    Given a pull request, mirror the bug in the current form of the repository.

    Args:
        inst: The instance to mirror.
        repo: The repository to mirror the bug in.
        model: The model to use for bug generation.
    Returns:
        A list of patch files.
    """
    patch_files = []
    patch = PatchSet(inst[KEY_PATCH])

    def extract_output(output):
        code_block_pat = re.compile(r"^```python\s*\n([\s\S]*)^```\s*$", re.MULTILINE)
        if code_block_pat.search(output):
            output = output.split("```python", 1)[1]
            output = output.rsplit("```", 1)[0]
            output = output.strip()
            output = code_block_pat.sub("", output)
        return output

    metadata = {KEY_COST: 0, KEY_REWRITES: {}, KEY_RECOVER_STATUS: RECOVER_SUCCESS}
    for idx, file_diff in enumerate(patch):
        file_path = os.path.join(repo, file_diff.path)

        if file_diff.is_added_file and os.path.exists(file_path):
            os.remove(file_path)
            patch = get_patch(repo, reset_changes=True)
            if patch:
                patch_path = f"{inst[KEY_INSTANCE_ID]}_{idx}.diff"
                with open(patch_path, "w") as f:
                    f.write(patch)
                patch_files.append(patch_path)
            continue
        elif file_diff.is_removed_file:
            if not os.path.exists(os.path.dirname(file_path)):
                # Skip over re-adding removed file if the parent directory doesn't exist
                continue
            with open(file_path, "w") as f:
                # Write the removed lines to the file
                f.write(
                    "".join(
                        line.value
                        for hunk in file_diff
                        for line in hunk
                        if line.is_removed
                    )
                )
            patch = get_patch(repo, reset_changes=True)
            if patch:
                patch_path = f"{inst[KEY_INSTANCE_ID]}_{idx}.diff"
                with open(patch_path, "w") as f:
                    f.write(patch)
                patch_files.append(patch_path)
            continue

        if not os.path.exists(file_path) or not file_path.endswith(".py"):
            # Skip over edits to files that don't exist or are not Python files
            continue
        file_content = open(file_path).read()

        # Call llm generation
        response = completion(
            model=model,
            messages=[
                {"role": "user", "content": RECOVERY_PROMPT},
                {"role": "user", "content": DEMO_PROMPT},
                {
                    "role": "user",
                    "content": TASK_PROMPT.format(file_content, str(file_diff)),
                },
            ],
            n=1,
            temperature=0,
            api_key=api_key,
        )

        # Perform rewrite
        cost = completion_cost(completion_response=response)
        metadata[KEY_COST] += cost
        metadata[INSTANCE_REF] = inst
        output = response.choices[0].message.content.strip()  # type: ignore
        output_extracted = extract_output(output)
        metadata[KEY_REWRITES][file_path] = {
            "output": output,
            "output_extracted": output_extracted,
            KEY_COST: cost,
        }
        with open(file_path, "w") as f:
            f.write(output_extracted)

        # Get patch from codebase
        try:
            patch = get_patch(repo, reset_changes=True)
            if not patch:
                raise ValueError("Patch is empty")
            patch_path = f"{inst[KEY_INSTANCE_ID]}_{idx}.diff"
            with open(patch_path, "w") as f:
                f.write(patch)
            patch_files.append(patch_path)
        except Exception as e:
            logger.error(f"Failed to get patch: {e}")
            continue

    # Save logs
    if log_path is None:
        log_path = LOG_DIR_BUG_GEN / repo / MIRROR_PR / inst[KEY_INSTANCE_ID]
    metadata_file = log_path / get_metadata_file_name(inst[KEY_PULL_NUM])
    ref_patch_file = log_path / f"ref__pr_{inst[KEY_PULL_NUM]}.diff"
    with open(metadata_file, "w") as f:
        if len(patch_files) == 0:
            metadata[KEY_RECOVER_STATUS] = RECOVER_FAIL
        json.dump(metadata, f, indent=4)
    with open(ref_patch_file, "w") as f:
        f.write(inst[KEY_PATCH])

    return patch_files


def should_process_instance(inst, repo, redo_existing, redo_skipped):
    """
    Determine if an instance should be processed based on existing metadata.
    """
    log_path = LOG_DIR_BUG_GEN / repo / MIRROR_PR / inst[KEY_INSTANCE_ID]
    metadata_file = log_path / get_metadata_file_name(inst[KEY_PULL_NUM])

    if not os.path.exists(metadata_file):
        return True, None

    metadata = json.load(open(metadata_file))
    recover_status = metadata[KEY_RECOVER_STATUS]

    if redo_existing and redo_skipped:
        return True, recover_status
    elif redo_existing and recover_status != RECOVER_SKIPPED:
        return True, recover_status
    elif redo_skipped and recover_status == RECOVER_SKIPPED:
        return True, recover_status

    return False, recover_status


def process_single_instance(
    inst, repo, model, api_key=None, max_files=8, max_lines=500, max_file_lines=10000
):
    """Process a single instance with its own working directory."""
    global this_worker_id
    temp_dir = worker_tempdirs[this_worker_id]
    original_dir = os.getcwd()
    try:
        log_path = (
            (LOG_DIR_BUG_GEN / repo / MIRROR_PR / inst[KEY_INSTANCE_ID])
            .resolve()
            .absolute()
        )
        metadata_file = log_path / get_metadata_file_name(inst[KEY_PULL_NUM])
        os.makedirs(log_path, exist_ok=True)

        os.chdir(temp_dir)
        registry.get(repo).clone()

        # Check if we should attempt recovery
        attempt_recovery, reason = should_attempt_recovery(
            inst, repo, max_files, max_lines, max_file_lines
        )
        if not attempt_recovery:
            with open(metadata_file, "w") as f:
                json.dump(
                    {
                        KEY_RECOVER_STATUS: RECOVER_SKIPPED,
                        KEY_SKIP_REASON: reason,
                    },
                    f,
                    indent=4,
                )
            return "skipped"

        # Attempt to apply patch directly to repo
        bug_file = log_path / f"{PREFIX_BUG}__pr_{inst[KEY_PULL_NUM]}.diff"
        direct_patch = f"{inst[KEY_INSTANCE_ID]}.diff"
        with open(direct_patch, "w") as f:
            f.write(inst[KEY_PATCH])
        if apply_patches(repo, [direct_patch]):
            with open(bug_file, "w") as f:
                f.write(inst[KEY_PATCH])
            with open(metadata_file, "w") as f:
                json.dump(
                    {
                        KEY_RECOVER_STATUS: RECOVER_SUCCESS,
                        KEY_COST: 0,
                        KEY_REWRITES: {},
                        "direct_patch": True,
                        INSTANCE_REF: inst,
                    },
                    f,
                    indent=4,
                )
            os.remove(direct_patch)
            return "recover_success"
        else:
            os.remove(direct_patch)

        # Attempt to perform recovery
        patch_files = recover_sweb_inst(
            inst, repo, model, api_key=api_key, log_path=log_path
        )

        if len(patch_files) == 0:
            return {"status": "recover_fail"}
        else:
            patch_merged = apply_patches(repo, patch_files)
            if patch_merged:
                with open(bug_file, "w") as f:
                    f.write(patch_merged)
                for patch_file in patch_files:
                    os.remove(patch_file)
                return "recover_success"
            else:
                return "recover_fail"
    except Exception as e:
        logger.error(f"Error processing instance {inst[KEY_INSTANCE_ID]}: {e}")
        logger.error(traceback.format_exc())
        return "recover_fail"
    finally:
        os.chdir(original_dir)


def init_worker():
    """
    When ProcessPoolExecutor workers are initialized, we
    """
    global this_worker_id, worker_tempdirs
    this_worker_id = int(current_process().name.split("-")[-1])
    worker_tempdirs[this_worker_id] = f"mirror_tmps/{str(uuid.uuid4())[:8]}"
    print(
        f"Initialized worker {this_worker_id} with temp dir {worker_tempdirs[this_worker_id]} (PID: {os.getpid()})"
    )
    os.makedirs(worker_tempdirs[this_worker_id], exist_ok=True)


def sweb_inst_to_rp(inst: dict) -> RepoProfile:
    owner, repo = inst["repo"].split("/")
    rps = [x for x in registry.values() if x.owner == owner and x.repo == repo]
    if len(rps) == 0:
        raise ValueError(
            f"{repo} not found in SWE-smith registry, create profile for repo under swesmith/profiles"
        )
    elif len(rps) > 1:
        print(f"Multiple profiles for {owner}/{repo} found")
        for i, rp in enumerate(rps):
            print(f"{i + 1}. {rp.commit}")
        idx = int(input("Enter index of RepoProfile to use: "))
        return rps[idx]
    return rps[0]


def main(
    sweb_insts_files: list,
    model: str,
    redo_existing: bool,
    redo_skipped: bool,
    api_key: str | None = None,
    num_processes: int = 1,
    max_files: int = 8,
    max_lines: int = 500,
    max_file_lines: int = 10000,
):
    global worker_tempdirs, this_worker_id

    assert not (redo_existing and redo_skipped), (
        "Cannot redo existing and skipped at the same time"
    )

    all_instances = []
    seen_repo_inst_ids = set()

    for sweb_insts_file in sweb_insts_files:
        if any([sweb_insts_file.endswith(ext) for ext in [".jsonl", ".jsonl.all"]]):
            file_instances = [json.loads(line) for line in open(sweb_insts_file)]
        elif sweb_insts_file.endswith(".json"):
            file_instances = json.load(open(sweb_insts_file))
        else:
            raise ValueError(
                f"Invalid file format for {sweb_insts_file}. Must be .json or .jsonl"
            )
        for inst in file_instances:
            inst[MIRROR_PR] = sweb_inst_to_rp(inst).repo_name
            repo_inst_id = (inst[MIRROR_PR], inst[KEY_INSTANCE_ID])
            if repo_inst_id in seen_repo_inst_ids:
                raise ValueError(f"Duplicate instance ID: {inst[KEY_INSTANCE_ID]}")
            seen_repo_inst_ids.add(repo_inst_id)
            all_instances.append(inst)
    print(f"Found {len(all_instances)} instances across {len(sweb_insts_files)} files")

    to_process = []
    already_completed = {RECOVER_SUCCESS: [], RECOVER_FAIL: [], RECOVER_SKIPPED: []}
    all_repos = set()
    repos_to_process = set()
    for inst in all_instances:
        should_process, status = should_process_instance(
            inst, inst[MIRROR_PR], redo_existing, redo_skipped
        )
        if should_process:
            to_process.append(inst)
        elif status:
            already_completed[status].append(inst)
        all_repos.add(inst[MIRROR_PR])
        if should_process:
            repos_to_process.add(inst[MIRROR_PR])
    print("Pre-processing report:")
    print(f"- Repos to process: {len(repos_to_process)}")
    print(f"- Instances to process: {len(to_process)}")
    print(
        f"- Already completed instances: {sum(len(v) for v in already_completed.values())}"
    )
    print(f"- All repos: {len(all_repos)}")
    print(f"  - Success: {len(already_completed[RECOVER_SUCCESS])}")
    print(f"  - Failed: {len(already_completed[RECOVER_FAIL])}")
    print(f"  - Skipped: {len(already_completed[RECOVER_SKIPPED])}")
    if not to_process:
        print("No instances to process. Exiting.")
        return

    num_processes = min(num_processes, len(to_process))
    print(f"Using {num_processes} processes")

    task_args = []
    for inst in to_process:
        task_args.append(
            (
                inst,
                inst[MIRROR_PR],
                model,
                api_key,
                max_files,
                max_lines,
                max_file_lines,
            )
        )

    pbar = tqdm(total=len(task_args))

    results = {"skipped": 0, "recover_success": 0, "recover_fail": 0}
    if num_processes > 1:
        worker_pids = {}

        with ProcessPoolExecutor(
            max_workers=num_processes, initializer=init_worker
        ) as pool:
            try:
                futures = [
                    pool.submit(process_single_instance, *args) for args in task_args
                ]

                # Store worker process PIDs
                for executor in pool._processes.values():
                    worker_pids[executor.pid] = executor
                print(f"Worker PIDs: {list(worker_pids.keys())}")

                for future in as_completed(futures):
                    result = future.result()
                    if result in results:
                        results[result] += 1
                    else:
                        print(f"Unknown result: {result}")
                    pbar.update(1)
            except KeyboardInterrupt:
                print("\nKeyboard interrupt. Forcefully killing all workers...")
                print(f"Partial results: {results}")
                for pid in worker_pids:
                    try:
                        print(f"Sending SIGKILL to worker PID {pid}")
                        os.kill(pid, signal.SIGKILL)
                    except OSError as e:
                        print(f"Error killing process {pid}: {e}")
                pool.shutdown(wait=False)
                raise KeyboardInterrupt
            finally:
                for temp_dir in worker_tempdirs.values():
                    if os.path.exists(temp_dir):
                        shutil.rmtree(temp_dir)
    else:
        # Single process mode
        worker_tempdirs = {0: f"tmp_{str(uuid.uuid4())[:8]}"}
        os.makedirs(worker_tempdirs[0], exist_ok=True)
        this_worker_id = 0
        for args in task_args:
            result = process_single_instance(*args)
            if result in results:
                results[result] += 1
            pbar.update(1)
        if os.path.exists(worker_tempdirs[0]):
            shutil.rmtree(worker_tempdirs[0])

    pbar.close()

    # Update results with already completed instances if needed
    if not redo_existing and not redo_skipped:
        results["skipped"] += len(already_completed[RECOVER_SKIPPED])
        results["recover_success"] += len(already_completed[RECOVER_SUCCESS])
        results["recover_fail"] += len(already_completed[RECOVER_FAIL])
    elif redo_existing and not redo_skipped:
        results["skipped"] += len(already_completed[RECOVER_SKIPPED])
    elif redo_skipped and not redo_existing:
        results["recover_success"] += len(already_completed[RECOVER_SUCCESS])
        results["recover_fail"] += len(already_completed[RECOVER_FAIL])

    print(f"\nFinal summary for ({len(all_instances)} instances)")
    print(f"- Skipped {results['skipped']}")
    print(f"- Recovery Success: {results['recover_success']}")
    print(f"- Recovery Fail: {results['recover_fail']}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Given a pull request, mirror the bug in a repository."
    )
    parser.add_argument(
        "sweb_insts_files",
        type=str,
        nargs="+",
        help="Paths to one or more swe-bench-task-instances.json[l] files.",
    )
    parser.add_argument(
        "--model",
        type=str,
        help="Model to use for bug generation",
        default="openai/gpt-4o",
    )
    parser.add_argument(
        "--redo_existing",
        action="store_true",
        help="Whether to redo existing bugs",
        default=False,
    )
    parser.add_argument(
        "--redo_skipped",
        action="store_true",
        help="Whether to redo bugs skipped due to failing recovery criteria",
        default=False,
    )
    parser.add_argument(
        "-n",
        "--num_processes",
        type=int,
        default=1,
    )
    parser.add_argument(
        "-f",
        "--max_files",
        type=int,
        default=8,
        help="Maximum number of files that can be changed for recovery attempt (default: 8)",
    )
    parser.add_argument(
        "-l",
        "--max_lines",
        type=int,
        default=500,
        help="Maximum total lines that can be changed for recovery attempt (default: 500)",
    )
    parser.add_argument(
        "-m",
        "--max_file_lines",
        type=int,
        default=10000,
        help="Maximum lines in a single changed file for recovery attempt (default: 10000)",
    )
    args = parser.parse_args()
    main(**vars(args))



"""
Purpose: Collect all the patches into a single json file that can be fed into swesmith.harness.valid

Usage: python -m swesmith.bug_gen.collect_patches logs/bug_gen/<repo>

NOTE: Must be with respect to a logs/bug_gen/<...>/ directory
"""

import argparse
import os
import json
from pathlib import Path

from swebench.harness.constants import KEY_INSTANCE_ID
from swesmith.constants import LOG_DIR_BUG_GEN, KEY_PATCH, PREFIX_BUG


def main(bug_gen_path: str | Path, bug_type: str = "all", num_bugs: int = -1):
    """
    Collect all the patches into a single json file that can be fed into swebench.harness.valid
    :param repo_path: Path to the bug_gen logs.
    :param bug_type: Type of patches to collect. (default: all)
    :param num_bugs: Number of bugs to collect. (default: all)
    """
    bug_gen_path = Path(bug_gen_path)
    if not bug_gen_path.resolve().is_relative_to((Path() / LOG_DIR_BUG_GEN).resolve()):
        print(
            f"Warning: {bug_gen_path} may not point to a bug_gen log directory (should be in {(Path() / LOG_DIR_BUG_GEN).resolve()})."
        )

    repo = bug_gen_path.name

    patches = []
    prefix = f"{PREFIX_BUG}__"
    if bug_type != "all":
        prefix += bug_type + "_"
    exit_loop = False
    for root, _, files in os.walk(bug_gen_path):
        for file in files:
            if file.startswith(prefix) and file.endswith(".diff"):
                bug_type_and_uuid = file.split(f"{PREFIX_BUG}__")[-1].split(".diff")[0]
                instance_id = f"{repo}.{bug_type_and_uuid}"
                patch = {}

                # Add metadata if it exists
                metadata_file = f"metadata__{bug_type_and_uuid}.json"
                if os.path.exists(os.path.join(root, metadata_file)):
                    patch.update(json.load(open(os.path.join(root, metadata_file))))

                # Add necessary bug patch information
                patch.update(
                    {
                        KEY_INSTANCE_ID: instance_id,
                        KEY_PATCH: open(os.path.join(root, file), "r").read(),
                        "repo": repo,
                    }
                )
                patches.append(patch)
                if num_bugs != -1 and len(patches) >= num_bugs:
                    exit_loop = True
                    break
        if exit_loop:
            break

    bug_patches_file = (
        bug_gen_path.parent / f"{bug_gen_path.name}_{bug_type}_patches.json"
    )
    if num_bugs != -1:
        bug_patches_file = bug_patches_file.with_name(
            bug_patches_file.stem + f"_n{num_bugs}" + bug_patches_file.suffix
        )
    if len(patches) > 0:
        with open(bug_patches_file, "w") as f:
            f.write(json.dumps(patches, indent=4))
        print(f"Saved {len(patches)} patches to {bug_patches_file}")
    else:
        print(f"No patches found for `{bug_type}` in {bug_gen_path}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Collect all the patches into a single json file that can be fed into swesmith.harness.valid"
    )
    parser.add_argument("bug_gen_path", help="Path to the bug_gen logs.")
    parser.add_argument(
        "--type",
        dest="bug_type",
        type=str,
        help="Type of patches to collect. (default: all)",
        default="all",
    )
    parser.add_argument(
        "-n",
        "--num_bugs",
        type=int,
        help="Number of bugs to collect. (default: all)",
        default=-1,
    )
    args = parser.parse_args()
    main(**vars(args))



import re

from dataclasses import dataclass, field
from swesmith.constants import ENV_NAME, KEY_PATCH
from swebench.harness.constants import TestStatus
from swesmith.profiles.base import RepoProfile, registry
from swesmith.profiles.utils import X11_DEPS
from unidiff import PatchSet


@dataclass
class JavaScriptProfile(RepoProfile):
    """
    Profile for JavaScript repositories.
    """

    exts: list[str] = field(default_factory=lambda: [".js"])

    def extract_entities(
        self,
        dirs_exclude: list[str] = None,
        dirs_include: list[str] = [],
        exclude_tests: bool = True,
        max_entities: int = -1,
    ) -> list:
        """
        Override to exclude JavaScript build artifacts by default.

        JavaScript projects often have build/dist directories that contain
        transpiled/bundled code. We should only analyze source files.
        """
        if dirs_exclude is None:
            # Default exclusions for JavaScript projects
            dirs_exclude = [
                "dist",
                "build",
                "node_modules",
                "coverage",
                ".next",
                "out",
                "examples",
                "docs",
                "bin",
            ]

        return super().extract_entities(
            dirs_exclude=dirs_exclude,
            dirs_include=dirs_include,
            exclude_tests=exclude_tests,
            max_entities=max_entities,
        )


def default_npm_install_dockerfile(mirror_name: str, node_version: str = "18") -> str:
    return f"""FROM node:{node_version}-bullseye
RUN apt update && apt install -y git  
RUN git clone https://github.com/{mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
"""


def parse_log_jest(log: str) -> dict[str, str]:
    """
    Parser for test logs generated with Jest. Assumes --verbose flag.

    Args:
        log (str): log content
    Returns:
        dict: test case to test status mapping
    """
    test_status_map = {}

    pattern = r"^\s*(✓|✕|○)\s(.+?)(?:\s\((\d+\s*m?s)\))?$"

    for line in log.split("\n"):
        match = re.match(pattern, line.strip())
        if match:
            status_symbol, test_name, _duration = match.groups()
            if status_symbol == "✓":
                test_status_map[test_name] = TestStatus.PASSED.value
            elif status_symbol == "✕":
                test_status_map[test_name] = TestStatus.FAILED.value
            elif status_symbol == "○":
                test_status_map[test_name] = TestStatus.SKIPPED.value
    return test_status_map


def parse_log_mocha(log: str) -> dict[str, str]:
    test_status_map = {}
    # Pattern for checkmark/x/dash style output
    # Note: Match both ✓ (U+2713) and ✔ (U+2714) checkmarks as different Mocha versions use different symbols
    pattern = r"^\s*([✓✔]|✖|-)\s(.+?)(?:\s\((\d+\s*m?s)\))?$"
    # Pattern for numbered failures like "1) test name" or "1) should solve..."
    fail_pattern = r"^\s*\d+\)\s+(.+?)(?:\s\((\d+\s*m?s)\))?$"
    for line in log.split("\n"):
        match = re.match(pattern, line.strip())
        if match:
            status_symbol, test_name, _duration = match.groups()
            if status_symbol in ("✓", "✔"):
                test_status_map[test_name] = TestStatus.PASSED.value
            elif status_symbol == "✖":
                test_status_map[test_name] = TestStatus.FAILED.value
            elif status_symbol == "-":
                test_status_map[test_name] = TestStatus.SKIPPED.value
        else:
            # Try numbered failure pattern
            fail_match = re.match(fail_pattern, line.strip())
            if fail_match:
                test_name = fail_match.group(1)
                test_status_map[test_name] = TestStatus.FAILED.value
    return test_status_map


def parse_log_vitest(log: str) -> dict[str, str]:
    test_status_map = {}
    patterns = [
        # Vitest uses ✓ for passing test files and ❯ for test files with failures
        (r"^✓\s+(.+?)(?:\s+\([\.\d]+ms\))?$", TestStatus.PASSED.value),
        (r"^❯\s+(.+?)(?:\s+\(.*?\))?$", TestStatus.FAILED.value),  # Failed test files
        (r"^✗\s+(.+?)(?:\s+\([\.\d]+ms\))?$", TestStatus.FAILED.value),
        (r"^○\s+(.+?)(?:\s+\([\.\d]+ms\))?$", TestStatus.SKIPPED.value),
        (r"^✓\s+(.+?)$", TestStatus.PASSED.value),
        (r"^✗\s+(.+?)$", TestStatus.FAILED.value),
        (r"^○\s+(.+?)$", TestStatus.SKIPPED.value),
    ]
    for line in log.split("\n"):
        for pattern, status in patterns:
            match = re.match(pattern, line.strip())
            if match:
                test_name = match.group(1).strip()
                # Normalize test file names: extract just the file path before parentheses
                # e.g., "test/foo.test.js (9 tests)" -> "test/foo.test.js"
                # or "test/foo.test.js (9 tests | 5 failed) 22ms" -> "test/foo.test.js"
                if "(" in test_name:
                    test_name = test_name.split("(")[0].strip()
                test_status_map[test_name] = status
                break

    return test_status_map


def parse_log_karma(log: str) -> dict[str, str]:
    """
    Parser for test logs generated by Karma (commonly used with Jasmine/Mocha).
    Since Karma doesn't output individual test names in a parseable way,
    we generate generic test entries based on the summary counts.
    """
    test_status_map = {}

    # Pattern for Karma final summary
    success_pattern = r"Executed\s+(\d+)\s+of\s+\d+\s+SUCCESS"
    failed_pattern = r"Executed\s+\d+\s+of\s+\d+\s+\((\d+)\s+FAILED\)"
    skipped_pattern = r"Executed\s+\d+\s+of\s+(\d+)\s+\((\d+)\s+skipped\)"

    passed_count = 0
    failed_count = 0
    skipped_count = 0

    for line in log.split("\n"):
        success_match = re.search(success_pattern, line)
        if success_match:
            passed_count = max(passed_count, int(success_match.group(1)))

        failed_match = re.search(failed_pattern, line)
        if failed_match:
            failed_count = max(failed_count, int(failed_match.group(1)))

        skipped_match = re.search(skipped_pattern, line)
        if skipped_match:
            skipped_count = max(skipped_count, int(skipped_match.group(2)))

    # Generate test entries
    for i in range(passed_count):
        test_status_map[f"karma_unit_test_{i + 1}"] = TestStatus.PASSED.value

    for i in range(failed_count):
        test_status_map[f"karma_unit_test_failed_{i + 1}"] = TestStatus.FAILED.value

    for i in range(skipped_count):
        test_status_map[f"karma_unit_test_skipped_{i + 1}"] = TestStatus.SKIPPED.value

    return test_status_map


def parse_log_jasmine(log: str) -> dict[str, str]:
    """
    Parser for standalone Jasmine CLI output.
    Format: "426 specs, 0 failures, 3 pending specs"
    """
    test_status_map = {}

    # Pattern for Jasmine summary: "X specs, Y failures, Z pending specs"
    pattern = r"(\d+)\s+specs?,\s+(\d+)\s+failures?(?:,\s+(\d+)\s+pending\s+specs?)?"

    for line in log.split("\n"):
        match = re.search(pattern, line)
        if match:
            total_specs = int(match.group(1))
            failures = int(match.group(2))
            pending = int(match.group(3)) if match.group(3) else 0

            passed = total_specs - failures - pending

            # Generate test entries
            for i in range(passed):
                test_status_map[f"jasmine_spec_{i + 1}"] = TestStatus.PASSED.value

            for i in range(failures):
                test_status_map[f"jasmine_spec_failed_{i + 1}"] = (
                    TestStatus.FAILED.value
                )

            for i in range(pending):
                test_status_map[f"jasmine_spec_pending_{i + 1}"] = (
                    TestStatus.SKIPPED.value
                )

            break  # Only process the first summary line

    return test_status_map


@dataclass
class ReactPDFee5c96b8(JavaScriptProfile):
    owner: str = "diegomura"
    repo: str = "react-pdf"
    commit: str = "ee5c96b80326ba4441b71be4c7a85ba9f61d4174"
    test_cmd: str = "./node_modules/.bin/vitest --no-color --reporter verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20-bullseye
RUN apt update && apt install -y pkg-config build-essential libpixman-1-0 libpixman-1-dev libcairo2-dev libpango1.0-dev libjpeg-dev libgif-dev librsvg2-dev
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN yarn install
"""

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        for line in log.split("\n"):
            for pattern, status in [
                (r"^\s*✓\s(.*)\s\d+ms", TestStatus.PASSED.value),
                (r"^\s*✗\s(.*)\s\d+ms", TestStatus.FAILED.value),
                (r"^\s*✖\s(.*)", TestStatus.FAILED.value),
                (r"^\s*✓\s(.*)", TestStatus.PASSED.value),
            ]:
                match = re.match(pattern, line)
                if match:
                    test_name = match.group(1).strip()
                    test_status_map[test_name] = status
                    break
        return test_status_map


@dataclass
class Markeddbf29d91(JavaScriptProfile):
    owner: str = "markedjs"
    repo: str = "marked"
    commit: str = "dbf29d9171a28da21f06122d643baf4e5d4266d4"
    test_cmd: str = "NO_COLOR=1 node --test"

    @property
    def dockerfile(self):
        return f"""FROM node:24-bullseye
RUN apt update && apt install -y git {X11_DEPS}
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
RUN npm test
"""

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        fail_pattern = r"^\s*✖\s(.*?)\s\([\.\d]+ms\)"
        pass_pattern = r"^\s*✔\s(.*?)\s\([\.\d]+ms\)"
        for line in log.split("\n"):
            fail_match = re.match(fail_pattern, line)
            if fail_match:
                test = fail_match.group(1)
                test_status_map[test.strip()] = TestStatus.FAILED.value
            else:
                pass_match = re.match(pass_pattern, line)
                if pass_match:
                    test = pass_match.group(1)
                    test_status_map[test.strip()] = TestStatus.PASSED.value
        return test_status_map


@dataclass
class Babel2ea3fc8f(JavaScriptProfile):
    owner: str = "babel"
    repo: str = "babel"
    commit: str = "2ea3fc8f9b33a911840f17fbc407e7bfae2ed66f"
    test_cmd: str = "yarn jest --verbose"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return f"""FROM node:20-bullseye
RUN apt update && apt install -y git
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN make bootstrap
RUN make build
"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)

    def get_test_cmd(self, instance: dict, f2p_only: bool = False):
        if KEY_PATCH not in instance:
            return self.test_cmd, []
        test_folders = []
        for f in PatchSet(instance[KEY_PATCH]):
            parts = f.path.split("/")
            if len(parts) >= 2 and parts[0] == "packages":
                test_folders.append("/".join(parts[:2]))
        return f"{self.test_cmd} {' '.join(test_folders)}", test_folders


@dataclass
class GithubReadmeStats3e974011(JavaScriptProfile):
    owner: str = "anuraghazra"
    repo: str = "github-readme-stats"
    commit: str = "3e97401177143bb35abb42279a13991cbd584ca3"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name)

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Mongoose5f57a5bb(JavaScriptProfile):
    owner: str = "Automattic"
    repo: str = "mongoose"
    commit: str = "5f57a5bbb2e8dfed8d04be47cdd17728633c44c1"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name)

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Axiosef36347f(JavaScriptProfile):
    owner: str = "axios"
    repo: str = "axios"
    commit: str = "ef36347fb559383b04c755b07f1a8d11897fab7f"
    test_cmd: str = "npm run test:mocha -- --verbose"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name)

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Async23dbf76a(JavaScriptProfile):
    owner: str = "caolan"
    repo: str = "async"
    commit: str = "23dbf76aeb04c7c3dd56276115b277e3fa9dd5cc"
    test_cmd: str = "npm run mocha-node-test -- --verbose"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name)

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Expressef5f2e13(JavaScriptProfile):
    owner: str = "expressjs"
    repo: str = "express"
    commit: str = "ef5f2e13ef64a1575ce8c2d77b180d593644ccfa"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name)

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Dayjsc8a26460(JavaScriptProfile):
    owner: str = "iamkun"
    repo: str = "dayjs"
    commit: str = "c8a26460d89a2ee9a7d3b9cafa124ea856ee883f"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name)

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Svelte6c9717a9(JavaScriptProfile):
    owner: str = "sveltejs"
    repo: str = "svelte"
    commit: str = "6c9717a91f2f6ae10641d1cf502ba13d227fbe45"
    test_cmd: str = "pnpm test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-bullseye
RUN apt update && apt install -y git
RUN npm install -g pnpm@10.4.0
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN pnpm install
RUN pnpm playwright install chromium
RUN pnpm exec playwright install-deps
"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_vitest(log)


@dataclass
class Commanderjs395cf714(JavaScriptProfile):
    owner: str = "tj"
    repo: str = "commander.js"
    commit: str = "395cf7145fe28122f5a69026b310e02df114f907"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name, node_version="20")

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Wretch661865a6(JavaScriptProfile):
    owner: str = "elbywan"
    repo: str = "wretch"
    commit: str = "661865a6642f6be26e742a90a3e0a9b9bd5542ff"
    test_cmd: str = "npm run test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:22-bullseye
RUN apt update && apt install -y git
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
RUN npm run build
"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Html5Boilerplateac08a17c(JavaScriptProfile):
    owner: str = "h5bp"
    repo: str = "html5-boilerplate"
    commit: str = "ac08a17cb60a975336664c0090657a3e593f686e"
    test_cmd: str = "npm run test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:22-bullseye
RUN apt update && apt install -y git
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm ci
"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class HighlightJS5697ae51(JavaScriptProfile):
    owner: str = "highlightjs"
    repo: str = "highlight.js"
    commit: str = "5697ae5187746c24732e62cd625f3f83004a44ce"
    test_cmd: str = "npm run test -- --verbose"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multimodal"}
    )

    @property
    def dockerfile(self):
        return f"""FROM node:22-bullseye
RUN apt update && apt install -y git
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
RUN npm run build
"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Prism31b467fa(JavaScriptProfile):
    owner: str = "PrismJS"
    repo: str = "prism"
    commit: str = "31b467fa7c92c5ce90c3e7c6c8fe2b8a946d9484"
    test_cmd: str = "npm run test"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multimodal"}
    )

    @property
    def dockerfile(self):
        return f"""FROM node:22-bullseye
RUN apt update && apt install -y git
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm ci
RUN npm run build
"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class ChromaJS498427ea(JavaScriptProfile):
    owner: str = "gka"
    repo: str = "chroma.js"
    commit: str = "498427eafc2e987a3751f8d5fe0612fa7a4a76ec"
    test_cmd: str = "npm run test -- --run"

    @property
    def dockerfile(self):
        return f"""FROM node:22-bullseye
RUN apt update && apt install -y git
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
RUN npm run build
"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_vitest(log)


@dataclass
class Colorfef7b619(JavaScriptProfile):
    owner: str = "Qix-"
    repo: str = "color"
    commit: str = "fef7b619edd678455595b9b6a10780f13b58d285"
    test_cmd: str = "npm run test -- --verbose"

    @property
    def image_name(self) -> str:
        # Note: "-" followed by a "_" is not allowed in Docker image names
        return f"{self.org_dh}/swesmith.{self.arch}.{self.owner.replace('-', '_')}_1776_{self.repo}.{self.commit[:8]}".lower()

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name, node_version="22")

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Qd180f4a0(JavaScriptProfile):
    owner: str = "kriskowal"
    repo: str = "q"
    commit: str = "d180f4a0b22499607ac750b56766c8829d6bff43"
    test_cmd: str = "npm run test -- --verbose --reporter spec"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name, node_version="22")

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class ImmutableJS879adab5(JavaScriptProfile):
    owner: str = "immutable-js"
    repo: str = "immutable-js"
    commit: str = "879adab5ea333a5ca341635bcf799c3b8f9e7559"
    test_cmd: str = "npm run test -- --verbose"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name, node_version="22")

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class ThreeJS73b3f248(JavaScriptProfile):
    owner: str = "mrdoob"
    repo: str = "three.js"
    commit: str = "73b3f248016fb73f2fe71da8616cdd7e20386f81"
    test_cmd: str = "npm run test -- --verbose"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name, node_version="22")

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Echarts6be0e145(JavaScriptProfile):
    owner: str = "apache"
    repo: str = "echarts"
    commit: str = "6be0e145946db37824c8635067b8b7b23c547b74"
    test_cmd: str = "npm run test -- --verbose"

    @property
    def dockerfile(self):
        return default_npm_install_dockerfile(self.mirror_name, node_version="22")

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Draggable8a1eed57(JavaScriptProfile):
    owner: str = "Shopify"
    repo: str = "draggable"
    commit: str = "8a1eed57f3ab2dff9371e8ce60fb39ac85871e8d"
    test_cmd: str = "yarn test --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN yarn install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Reactslick97442318(JavaScriptProfile):
    owner: str = "akiran"
    repo: str = "react-slick"
    commit: str = "97442318e9a442bd4a84eb25133ef62087f98232"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

# Install system dependencies
RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Set the default command
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Pdfmake719e7314(JavaScriptProfile):
    owner: str = "bpampuch"
    repo: str = "pdfmake"
    commit: str = "719e73140cce75a792f7f419c27fc33a230e73d2"
    test_cmd: str = "npm run test"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

# Install system dependencies
RUN apt-get update && apt-get install -y \
    git \
    && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Multerb6e4b1f6(JavaScriptProfile):
    owner: str = "expressjs"
    repo: str = "multer"
    commit: str = "b6e4b1f6abb85673e9307b42368b3e7bfb1fc63b"
    test_cmd: str = "npm test -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim
RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Pdfkitd0108157(JavaScriptProfile):
    owner: str = "foliojs"
    repo: str = "pdfkit"
    commit: str = "d0108157f13d763ad5287a2293436b5a1aecf055"
    test_cmd: str = "yarn test --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

# Install system dependencies
RUN apt-get update && apt-get install -y \
    git \
    build-essential \
    libcairo2-dev \
    libpango1.0-dev \
    libjpeg-dev \
    libgif-dev \
    librsvg2-dev \
    && rm -rf /var/lib/apt/lists/*

WORKDIR /{ENV_NAME}

# Enable corepack to use the yarn version specified in package.json
RUN corepack enable

# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN yarn install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Mathjs04e6e2d7(JavaScriptProfile):
    owner: str = "josdejong"
    repo: str = "mathjs"
    commit: str = "04e6e2d7a949d6ddc7d7139bf1e3a88e6fe5365b"
    test_cmd: str = "npm run test:src -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

# Install git and other system dependencies if needed
RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Build the project (as it seems to have a build step that generates lib/ which might be needed for tests)
RUN npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)  # Default fallback


@dataclass
class Jqueryc28c26ae(JavaScriptProfile):
    owner: str = "jquery"
    repo: str = "jquery"
    commit: str = "c28c26aef0b3238f578690d73703382951cb355d"
    test_cmd: str = "npm run test:browserless -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install system dependencies
RUN apt-get update && apt-get install -y \
    git \
    python3 \
    make \
    g++ \
    && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Default command
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_qunit(log)


@dataclass
class Koa0a6afa5a(JavaScriptProfile):
    owner: str = "koajs"
    repo: str = "koa"
    commit: str = "0a6afa5a6107c0c8baf4722e29de7566f33d1651"
    test_cmd: str = "node --test"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Layuiabdb748b(JavaScriptProfile):
    owner: str = "layui"
    repo: str = "layui"
    commit: str = "abdb748b5cc792c394fbdf56daa2727af1846488"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install git
RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Mocha410ce0d2(JavaScriptProfile):
    owner: str = "mochajs"
    repo: str = "mocha"
    commit: str = "410ce0d2a0f799aaca2c0bc627294d70c62dd3f4"
    test_cmd: str = "npm run test-node:unit"

    @property
    def dockerfile(self):
        return f"""FROM node:22-slim

# Install git
RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Set the default command
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Reactnativeweba9de220b(JavaScriptProfile):
    owner: str = "necolas"
    repo: str = "react-native-web"
    commit: str = "a9de220ba9e65bdea540fb5322ffb1da2b0bf442"
    test_cmd: str = "npm run unit:dom -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18

# Install system dependencies
RUN apt-get update && apt-get install -y \
    git \
    && rm -rf /var/lib/apt/lists/*

# Set working directory

# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Set default command
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Piskel51373322(JavaScriptProfile):
    owner: str = "piskelapp"
    repo: str = "piskel"
    commit: str = "513733227695da58780a4df30f44e4af9f85b1a6"
    test_cmd: str = "npm run unit-tests"

    @property
    def dockerfile(self):
        return f"""FROM node:18-bullseye-slim

# Install system dependencies for Playwright and Puppeteer
RUN apt-get update && apt-get install -y \
    git \
    libnss3 \
    libdbus-1-3 \
    libatk1.0-0 \
    libatk-bridge2.0-0 \
    libcups2 \
    libdrm2 \
    libxcomposite1 \
    libxdamage1 \
    libxext6 \
    libxfixes3 \
    libxrandr2 \
    libgbm1 \
    libasound2 \
    libpangocairo-1.0-0 \
    libx11-6 \
    libxkbcommon0 \
    libpango-1.0-0 \
    libcairo2 \
    && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Install Playwright browsers and their dependencies
RUN npx playwright install --with-deps chromium

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_karma(log)


@dataclass
class Reduxsagaa4ace10d(JavaScriptProfile):
    owner: str = "redux-saga"
    repo: str = "redux-saga"
    commit: str = "a4ace10dc3ff182828cd3ee7469f6667e08ceb62"
    test_cmd: str = "yarn test --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

# Install git for cloning and patching
RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies using yarn (yarn.lock is present)
RUN yarn install --frozen-lockfile

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Riot32aecfaa(JavaScriptProfile):
    owner: str = "riot"
    repo: str = "riot"
    commit: str = "32aecfaa424609ba35829f645138f182f3273dce"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y \
    git \
    make \
    procps \
    && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Svgoc06d8f68(JavaScriptProfile):
    owner: str = "svg"
    repo: str = "svgo"
    commit: str = "c06d8f6899788defae9594537063c2f4307803e4"
    test_cmd: str = "yarn cross-env NODE_OPTIONS=--experimental-vm-modules jest --maxWorkers=4 --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install git for cloning the repository
RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Enable corepack to use the yarn version specified in package.json and install dependencies
RUN corepack enable && yarn install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Bruno80e09d1a(JavaScriptProfile):
    owner: str = "usebruno"
    repo: str = "bruno"
    commit: str = "80e09d1a267ed2283e6d58a643800d3d632372a7"
    test_cmd: str = "npm test --workspaces --if-present -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:22-slim

RUN apt-get update && apt-get install -y git python3 make g++ && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm run setup

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Webtorrentfd8f39e1(JavaScriptProfile):
    owner: str = "webtorrent"
    repo: str = "webtorrent"
    commit: str = "fd8f39e1560c5ae5db6b12153077877f0f33b076"
    test_cmd: str = "npx tape test/*.js test/node/*.js"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install system dependencies for native modules
RUN apt-get update && apt-get install -y \
    python3 \
    make \
    g++ \
    git \
    && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Whydidyourender3ec3512d(JavaScriptProfile):
    owner: str = "welldone-software"
    repo: str = "why-did-you-render"
    commit: str = "3ec3512d750c49448fe2241e26d05db9e42f0c21"
    test_cmd: str = "yarn test --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install system dependencies
RUN apt-get update && apt-get install -y \
    git \
    && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN yarn install

# Keep the container running
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Eleventye9a16667(JavaScriptProfile):
    owner: str = "11ty"
    repo: str = "eleventy"
    commit: str = "e9a16667cbf44226d4dc88ac18241003e05908d2"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

CMD ["npm", "test"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Workbox1893b3f6(JavaScriptProfile):
    owner: str = "GoogleChrome"
    repo: str = "workbox"
    commit: str = "1893b3f6ca3d82338f18acc84309f2f38fc67292"
    test_cmd: str = "npm run test_node -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install system dependencies required for building some native modules and git for cloning
RUN apt-get update && apt-get install -y git python3 make g++ && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies and build the project (required for tests to find built modules)
RUN npm ci && npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Habiticae0af620b(JavaScriptProfile):
    owner: str = "HabitRPG"
    repo: str = "habitica"
    commit: str = "e0af620b4045d46dffb4c22ea01f95ba8a8af009"
    test_cmd: str = "npm run test:api:unit -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:20

RUN apt-get update && apt-get install -y git python3 build-essential && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# The postinstall script in package.json handles:
# 1. gulp build
# 2. cd website/client && npm install
# We need to ensure dependencies for the main app are installed first.
# Also, Habitica expects a config.json file.
RUN cp config.json.example config.json
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Modernizr1d4c9cee(JavaScriptProfile):
    owner: str = "Modernizr"
    repo: str = "Modernizr"
    commit: str = "1d4c9cee1f358f50c31be9a1f247e1153ed9143c"
    test_cmd: str = "npm test -- --verbose --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

# Install system dependencies including git and chromium for puppeteer/mocha-headless-chrome
RUN apt-get update && apt-get install -y \
    git \
    wget \
    gnupg \
    ca-certificates \
    chromium \
    --no-install-recommends && \
    rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies, skipping puppeteer browser download since we use system chromium
ENV PUPPETEER_SKIP_DOWNLOAD=true
ENV CHROME_PATH=/usr/bin/chromium
ENV PUPPETEER_EXECUTABLE_PATH=/usr/bin/chromium
RUN npm install

# Set CMD
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Falcor39d64776(JavaScriptProfile):
    owner: str = "Netflix"
    repo: str = "falcor"
    commit: str = "39d64776cf9d87781b2791615dcbae73b2bcd2e1"
    test_cmd: str = "npm run test:only -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install --legacy-peer-deps

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Pm2ff1ca974(JavaScriptProfile):
    owner: str = "Unitech"
    repo: str = "pm2"
    commit: str = "ff1ca974afada8730aa55f8ed1df40e700cedbcb"
    test_cmd: str = "npm run test:unit -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git procps bc python3 && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Audiobookshelf626596b1(JavaScriptProfile):
    owner: str = "advplyr"
    repo: str = "audiobookshelf"
    commit: str = "626596b192013ba9f5a011dd110e288124c95ebe"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-bullseye-slim

RUN apt-get update && apt-get install -y \
    git \
    python3 \
    make \
    g++ \
    ffmpeg \
    && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install root dependencies
RUN npm ci

# Install client dependencies and build client
RUN cd client && npm ci && npm run generate

# Ensure we are back in root
WORKDIR /{ENV_NAME}

CMD ["npm", "start"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Sailsffebacc5(JavaScriptProfile):
    owner: str = "balderdashy"
    repo: str = "sails"
    commit: str = "ffebacc58c27f878c9373702bc3a3f91a02bca0c"
    test_cmd: str = "npm run custom-tests -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Vuebootstrapvue9a246f45(JavaScriptProfile):
    owner: str = "bootstrap-vue"
    repo: str = "bootstrap-vue"
    commit: str = "9a246f45fc813f161df291fc7d6197febf8afaf4"
    test_cmd: str = "yarn jest --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-bullseye-slim

RUN apt-get update && apt-get install -y git python3 make g++ && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN chmod u+x scripts/build.sh
RUN yarn install --frozen-lockfile

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Nodepostgresecff60dc(JavaScriptProfile):
    owner: str = "brianc"
    repo: str = "node-postgres"
    commit: str = "ecff60dc8aa0bd1ad5ea8f4623af0756a86dc110"
    test_cmd: str = "service postgresql start && sleep 5 && sudo -u postgres psql -c \"ALTER USER postgres WITH PASSWORD 'postgres';\" && export PGPASSWORD=postgres && export PGUSER=postgres && export PGHOST=localhost && yarn test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y \
    git \
    make \
    python3 \
    g++ \
    build-essential \
    libpq-dev \
    postgresql \
    postgresql-contrib \
    sudo \
    && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN yarn install
RUN yarn build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Claudecodetemplates734b8a50(JavaScriptProfile):
    owner: str = "davila7"
    repo: str = "claude-code-templates"
    commit: str = "734b8a50cc2cf55222643e32a3b205483e244747"
    test_cmd: str = "cd api && npm test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install && \
    cd cli-tool && npm install && \
    cd ../api && npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Jsemotionb882bcba(JavaScriptProfile):
    owner: str = "emotion-js"
    repo: str = "emotion"
    commit: str = "b882bcba85132554992e4bd49e94c95939bbf810"
    test_cmd: str = "yarn jest --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git python3 build-essential && rm -rf /var/lib/apt/lists/*

RUN corepack enable


RUN git clone --depth 1 https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN yarn install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Enzyme61e1b47c(JavaScriptProfile):
    owner: str = "enzymejs"
    repo: str = "enzyme"
    commit: str = "61e1b47c4bdc4509b2ac286c0d3ae3df172d26f0"
    test_cmd: str = "npm run react 16 && npm run test:only -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y \
    git \
    python3 \
    make \
    g++ \
    && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

ENV NODE_OPTIONS="--max-old-space-size=4096"
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Recoilc1b97f3a(JavaScriptProfile):
    owner: str = "facebookexperimental"
    repo: str = "Recoil"
    commit: str = "c1b97f3a0117cad76cbc6ab3cb06d89a9ce717af"
    test_cmd: str = "yarn relay"

    @property
    def dockerfile(self):
        return f"""FROM node:18

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN yarn install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Superagentcec26064(JavaScriptProfile):
    owner: str = "forwardemail"
    repo: str = "superagent"
    commit: str = "cec260643d6d8854865cf6a18997606be4b150f6"
    test_cmd: str = "./node_modules/.bin/mocha --require should --trace-warnings --throw-deprecation --reporter spec --slow 2000 --timeout 5000 --exit test/*.js test/node/*.js"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git build-essential python3 && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

RUN npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Supertest14d905dc(JavaScriptProfile):
    owner: str = "forwardemail"
    repo: str = "supertest"
    commit: str = "14d905dc313b7c050596342f833a52f0bc573c70"
    test_cmd: str = "npm test -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
CMD ["npm", "test"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Revealjsbecc9bd1(JavaScriptProfile):
    owner: str = "hakimel"
    repo: str = "reveal.js"
    commit: str = "becc9bd19e418b75027b541c41952105a1425c96"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install system dependencies for git and Puppeteer
RUN apt-get update && apt-get install -y \
    git \
    ca-certificates \
    fonts-liberation \
    libasound2 \
    libatk-bridge2.0-0 \
    libatk1.0-0 \
    libc6 \
    libcairo2 \
    libcups2 \
    libdbus-1-3 \
    libexpat1 \
    libfontconfig1 \
    libgbm1 \
    libgcc1 \
    libglib2.0-0 \
    libgtk-3-0 \
    libnspr4 \
    libnss3 \
    libpango-1.0-0 \
    libpangocairo-1.0-0 \
    libstdc++6 \
    libx11-6 \
    libx11-xcb1 \
    libxcb1 \
    libxcomposite1 \
    libxcursor1 \
    libxdamage1 \
    libxext6 \
    libxfixes3 \
    libxi6 \
    libxrandr2 \
    libxrender1 \
    libxss1 \
    libxtst6 \
    lsb-release \
    wget \
    xdg-utils \
    --no-install-recommends \
    && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Build the project (some tests might depend on built assets)
RUN npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Handsontablee71f0f42(JavaScriptProfile):
    owner: str = "handsontable"
    repo: str = "handsontable"
    commit: str = "e71f0f427c43eaaac9362d947270b8856a9766cd"
    test_cmd: str = "pnpm test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git python3 make g++ && rm -rf /var/lib/apt/lists/*

RUN npm install -g pnpm@10.12.2


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN pnpm install && pnpm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Joi481e270e(JavaScriptProfile):
    owner: str = "hapijs"
    repo: str = "joi"
    commit: str = "481e270e6c4ff8728d6fda248fd83f6ff70f7ed9"
    test_cmd: str = "npx lab -t 100 -a @hapi/code -L -Y -v"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Impressjsc9f6c674(JavaScriptProfile):
    owner: str = "impress"
    repo: str = "impress.js"
    commit: str = "c9f6c67457ceee5a011e554f67c447113640777d"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:18

RUN apt-get update && apt-get install -y \
    git \
    wget \
    gnupg \
    ca-certificates \
    libgconf-2-4 \
    libatk1.0-0 \
    libatk-bridge2.0-0 \
    libgdk-pixbuf2.0-0 \
    libgtk-3-0 \
    libgbm-dev \
    libnss3 \
    libxss1 \
    libasound2 \
    libxtst6 \
    xfonts-75dpi \
    xfonts-base \
    fonts-liberation \
    libappindicator3-1 \
    lsb-release \
    xdg-utils \
    libx11-xcb1 \
    libxcb-dri3-0 \
    --no-install-recommends \
    && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

RUN npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jasmine(log)


@dataclass
class Htmlwebpackplugin9a39db80(JavaScriptProfile):
    owner: str = "jantimon"
    repo: str = "html-webpack-plugin"
    commit: str = "9a39db807c09d8e6145e5047cfe2ec5e928e1dee"
    test_cmd: str = "npm run test:only -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install --legacy-peer-deps

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Backbonee8bc45ac(JavaScriptProfile):
    owner: str = "jashkenas"
    repo: str = "backbone"
    commit: str = "e8bc45acb0a8b035fe5a0d7338e1b2757681564f"
    test_cmd: str = "npx karma start --browsers ChromeHeadlessNoSandbox --single-run"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git chromium && rm -rf /var/lib/apt/lists/*

ENV CHROME_BIN=/usr/bin/chromium


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install && npm install karma-chrome-launcher --save-dev

# Add ChromeHeadlessNoSandbox launcher to karma.conf.js
RUN sed -i "s/customLaunchers: {{/customLaunchers: {{\\n        ChromeHeadlessNoSandbox: {{\\n            base: 'ChromeHeadless',\\n            flags: ['--no-sandbox']\\n        }},/" karma.conf.js

# Build debug-info.js which is required by tests
RUN npm run build-debug

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jasmine(log)


@dataclass
class Hyperapp5a113fa0(JavaScriptProfile):
    owner: str = "jorgebucaran"
    repo: str = "hyperapp"
    commit: str = "5a113fa00450302be9234e0a74ee634ed5574243"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Jsoneditor0319b213(JavaScriptProfile):
    owner: str = "josdejong"
    repo: str = "jsoneditor"
    commit: str = "0319b2131df47f1220d74e3ff174d5c02973ec7d"
    test_cmd: str = "npm test -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Uptimekuma5d955f95(JavaScriptProfile):
    owner: str = "louislam"
    repo: str = "uptime-kuma"
    commit: str = "5d955f954b60410cd2dc5370d429753de524a2ef"
    test_cmd: str = "npm run test-backend"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install system dependencies
RUN apt-get update && apt-get install -y \
    git \
    python3 \
    build-essential \
    iputils-ping \
    && rm -rf /var/lib/apt/lists/*


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Build the frontend
RUN npm run build

# Default command
CMD ["npm", "start"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Jsmarko24b9402c(JavaScriptProfile):
    owner: str = "marko-js"
    repo: str = "marko"
    commit: str = "24b9402cd54c3a74f200da0f79dd19350995a9ba"
    test_cmd: str = "env MARKO_DEBUG=1 ./node_modules/.bin/mocha --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git python3 build-essential && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install && npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Jsmdx00046053(JavaScriptProfile):
    owner: str = "mdx-js"
    repo: str = "mdx"
    commit: str = "000460532e6a558693cbe73c2ffdb8d6c098a07b"
    test_cmd: str = "npm run test-api --workspaces --if-present"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

RUN npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class PapaParseb10b87ef(JavaScriptProfile):
    owner: str = "mholt"
    repo: str = "PapaParse"
    commit: str = "b10b87ef8686c6f88299b50dd25e83606e9c36d4"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y \
    git \
    chromium \
    && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
RUN sed -i "s/'-f'/'-a', '[\"--no-sandbox\"]', '-f'/" tests/test.js

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Materialui1a233f88(JavaScriptProfile):
    owner: str = "mui"
    repo: str = "material-ui"
    commit: str = "1a233f8805ea20f456afd41165b1d6d9e22c0adb"
    test_cmd: str = "pnpm test:node run"

    @property
    def dockerfile(self):
        return f"""FROM node:22-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN npm install -g pnpm@10.25.0


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN pnpm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_vitest(log)


@dataclass
class Nightwatch54c8550c(JavaScriptProfile):
    owner: str = "nightwatchjs"
    repo: str = "nightwatch"
    commit: str = "54c8550c75a16c61827c0bad043c7ffa073a52e6"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install --ignore-scripts

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Nocke7418da2(JavaScriptProfile):
    owner: str = "nock"
    repo: str = "nock"
    commit: str = "e7418da29feb4a7bf0aa1612bfb9d32a4051651e"
    test_cmd: str = "npm test -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:20

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class NoVNCd44f7e04(JavaScriptProfile):
    owner: str = "novnc"
    repo: str = "noVNC"
    commit: str = "d44f7e04fc456844836c7c5ac911d0f4e8dd06e6"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

# Install system dependencies
RUN apt-get update && apt-get install -y \
    git \
    chromium \
    ca-certificates \
    --no-install-recommends && \
    rm -rf /var/lib/apt/lists/*

# Create a wrapper for chromium to always include --no-sandbox
RUN mv /usr/bin/chromium /usr/bin/chromium-orig && \
    echo -e '#!/bin/bash\\n/usr/bin/chromium-orig --no-sandbox "$@"' > /usr/bin/chromium && \
    chmod +x /usr/bin/chromium

# Set environment variables
ENV CHROME_BIN=/usr/bin/chromium
ENV TEST_BROWSER_NAME=ChromeHeadless


# Clone the repository
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

# Install dependencies
RUN npm install

# Command to keep container running
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class JsPDFe6cf03db(JavaScriptProfile):
    owner: str = "parallax"
    repo: str = "jsPDF"
    commit: str = "e6cf03db2499ef0a9ccc54b2aba45156c5b32b3c"
    test_cmd: str = "npm run test-node"

    @property
    def dockerfile(self):
        return rf"""FROM node:18

RUN apt-get update && apt-get install -y \
    git \
    libnss3 \
    libatk-bridge2.0-0 \
    libx11-xcb1 \
    libxcb-dri3-0 \
    libxcomposite1 \
    libxcursor1 \
    libxdamage1 \
    libxext6 \
    libxfixes3 \
    libxi6 \
    libxrandr2 \
    libxrender1 \
    libxtst6 \
    libcups2 \
    libdbus-1-3 \
    libexpat1 \
    libfontconfig1 \
    libgbm1 \
    libgcc1 \
    libglib2.0-0 \
    libgtk-3-0 \
    libpango-1.0-0 \
    libpangocairo-1.0-0 \
    libstdc++6 \
    libv4l-0 \
    libxkbcommon0 \
    libasound2 \
    wget \
    gnupg \
    --no-install-recommends \
    && wget -q -O - https://dl-ssl.google.com/linux/linux_signing_key.pub | apt-key add - \
    && sh -c 'echo "deb [arch=amd64] http://dl.google.com/linux/chrome/deb/ stable main" >> /etc/apt/sources.list.d/google.list' \
    && apt-get update \
    && apt-get install -y google-chrome-stable --no-install-recommends \
    && rm -rf /var/lib/apt/lists/*

ENV CHROME_BIN=/usr/bin/google-chrome-stable


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

# Inject custom launcher into karma.conf.js
RUN sed -i "s/browsers: \\['Chrome'\\]/browsers: ['ChromeHeadlessNoSandbox']/" test/unit/karma.conf.js && \
    sed -i "/reporters: \\[/i     customLaunchers: {{\\n      ChromeHeadlessNoSandbox: {{\\n        base: 'ChromeHeadless',\\n        flags: ['--no-sandbox', '--disable-setuid-sandbox']\\n      }}\\n    }}," test/unit/karma.conf.js

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Filepond38294959(JavaScriptProfile):
    owner: str = "pqina"
    repo: str = "filepond"
    commit: str = "38294959147229eb09126008fc09d295da4e30cd"
    test_cmd: str = "npm test -- --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Reacttransitiongroup2989b5b8(JavaScriptProfile):
    owner: str = "reactjs"
    repo: str = "react-transition-group"
    commit: str = "2989b5b87b4b4d1001f21c8efa503049ffb4fe8d"
    test_cmd: str = "npm run testonly"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install --legacy-peer-deps

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Reactmarkdownfda7fa56(JavaScriptProfile):
    owner: str = "remarkjs"
    repo: str = "react-markdown"
    commit: str = "fda7fa560bec901a6103e195f9b1979dab543b17"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Nodemondaad5c16(JavaScriptProfile):
    owner: str = "remy"
    repo: str = "nodemon"
    commit: str = "daad5c162919fa6abff53be16832bdf55f2204ad"
    test_cmd: str = "for FILE in test/**/*.test.js; do echo $FILE; TEST=1 ./node_modules/.bin/mocha --exit --timeout 30000 $FILE || true; sleep 1; done"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Evergreen9b774aee(JavaScriptProfile):
    owner: str = "segmentio"
    repo: str = "evergreen"
    commit: str = "9b774aee2d794f6cf2f73a054bd33066ca5898a9"
    test_cmd: str = "yarn jest --verbose"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN yarn install --frozen-lockfile

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Serverlessde62c71e(JavaScriptProfile):
    owner: str = "serverless"
    repo: str = "serverless"
    commit: str = "de62c71e30855eff688f032ff10b9ad22de13afc"
    test_cmd: str = "npm test -- --reporter spec"

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install
CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Jssqljs52e5649f(JavaScriptProfile):
    owner: str = "sql-js"
    repo: str = "sql.js"
    owner: str = "sql-js"
    repo: str = "sql.js"
    commit: str = "52e5649f3a3a2a46aa4ad58a79d118c22f56cf30"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM emscripten/emsdk:latest

RUN apt-get update && apt-get install -y git make python3 unzip curl libdigest-sha3-perl && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN npm install
RUN npm run build

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Jsonserverf5dfdaff(JavaScriptProfile):
    owner: str = "typicode"
    repo: str = "json-server"
    commit: str = "f5dfdaff725ecd5384b1f922b37757f023e13b63"
    test_cmd: str = "npm test"

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install
CMD ["npm", "start"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


@dataclass
class Webpack24e3c2d2(JavaScriptProfile):
    owner: str = "webpack"
    repo: str = "webpack"
    commit: str = "24e3c2d2c9f8c6d60810302b2ea70ed86e2863dc"
    test_cmd: str = (
        "yarn test:base --verbose --testMatch '<rootDir>/test/*.basictest.js'"
    )

    @property
    def dockerfile(self):
        return f"""FROM node:20-slim

RUN apt-get update && apt-get install -y git python3 build-essential && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}

RUN yarn install && yarn setup

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_jest(log)


@dataclass
class Ws726c3732(JavaScriptProfile):
    owner: str = "websockets"
    repo: str = "ws"
    commit: str = "726c3732b3e5319219ed73cac4826fd36917e2e1"
    test_cmd: str = "npm test -- --reporter spec"
    timeout: int = 300

    @property
    def dockerfile(self):
        return f"""FROM node:18-slim

RUN apt-get update && apt-get install -y git && rm -rf /var/lib/apt/lists/*


RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN npm install

CMD ["/bin/bash"]"""

    def log_parser(self, log: str) -> dict[str, str]:
        return parse_log_mocha(log)


# Register all JavaScript profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, JavaScriptProfile)
        and obj.__name__ != "JavaScriptProfile"
    ):
        registry.register_profile(obj)


# Register all JavaScript profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, JavaScriptProfile)
        and obj.__name__ != "JavaScriptProfile"
    ):
        registry.register_profile(obj)



"""
Purpose: Given a repository, procedurally generate a variety of bugs for functions/classes/objects in the repository.

Usage: python -m swesmith.bug_gen.procedural.generate \
    --repo <repo> \
    --commit <commit>
"""

import argparse
import json
import random
import shutil
import time

from pathlib import Path
from rich import print
from swesmith.bug_gen.utils import (
    apply_code_change,
    get_bug_directory,
    get_patch,
)
from swesmith.constants import (
    LOG_DIR_BUG_GEN,
    PREFIX_BUG,
    PREFIX_METADATA,
    BugRewrite,
    CodeEntity,
)
from swesmith.profiles import registry
from tqdm.auto import tqdm

from swesmith.bug_gen.procedural import MAP_EXT_TO_MODIFIERS
from swesmith.bug_gen.procedural.base import ProceduralModifier


def _process_candidate(
    candidate: CodeEntity, pm: ProceduralModifier, log_dir: Path, repo: str
):
    """
    Process a candidate by applying a given procedural modification to it.
    """
    # Get modified function
    bug: BugRewrite | None = pm.modify(candidate)
    if not bug:
        return False

    # Create artifacts
    bug_dir = get_bug_directory(log_dir, candidate)
    bug_dir.mkdir(parents=True, exist_ok=True)
    uuid_str = f"{pm.name}__{bug.get_hash()}"
    metadata_path = f"{PREFIX_METADATA}__{uuid_str}.json"
    bug_path = f"{PREFIX_BUG}__{uuid_str}.diff"

    with open(bug_dir / metadata_path, "w") as f:
        json.dump(bug.to_dict(), f, indent=2)
    apply_code_change(candidate, bug)
    patch = get_patch(repo, reset_changes=True)
    if patch:
        with open(bug_dir / bug_path, "w") as f:
            f.write(patch)
        return True
    return False


def main(
    repo: str,
    max_bugs: int,
    seed: int,
    interleave: bool = False,
    max_entities: int = -1,
    max_candidates: int = -1,
    timeout_seconds: int | None = None,
):
    random.seed(seed)
    total = 0
    start_time = time.time() if timeout_seconds is not None else None
    rp = registry.get(repo)
    rp.clone()
    entities = rp.extract_entities()

    def check_timeout():
        """Check if timeout has been reached. Returns True if should stop."""
        if start_time is None:
            return False
        elapsed = time.time() - start_time
        if elapsed >= timeout_seconds:
            print(
                f"\n[{repo}] TIMEOUT: Reached {timeout_seconds}s limit after {elapsed:.1f}s, stopping generation..."
            )
            return True
        return False

    # Apply entity sampling if limit is set and exceeded
    original_count = len(entities)
    if max_entities > 0 and original_count > max_entities:
        random.shuffle(entities)
        entities = entities[:max_entities]
        print(
            f"Found {original_count} entities in {repo}, sampled down to {max_entities} for efficiency."
        )
    else:
        print(f"Found {len(entities)} entities in {repo}.")

    log_dir = LOG_DIR_BUG_GEN / repo
    log_dir.mkdir(parents=True, exist_ok=True)
    print(f"Logging bugs to {log_dir}")

    def process_with_timeout():
        """Process candidates with timeout checking. Returns total bugs processed."""
        local_total = 0

        if interleave:
            # Build all (candidate, modifier) pairs upfront
            pairs = []
            for ext, pm_list in MAP_EXT_TO_MODIFIERS.items():
                for pm in pm_list:
                    candidates = [
                        x
                        for x in entities
                        if Path(x.file_path).suffix == ext and pm.can_change(x)
                    ]
                    if not candidates:
                        continue
                    print(f"[{repo}] Found {len(candidates)} candidates for {pm.name}.")

                    if max_bugs > 0 and len(candidates) > max_bugs:
                        candidates = random.sample(candidates, max_bugs)

                    # Add all pairs for this modifier
                    for candidate in candidates:
                        pairs.append((candidate, pm))

            # Shuffle all pairs to interleave modifiers
            random.shuffle(pairs)

            # Apply max_candidates limit if set
            original_pairs_len = len(pairs)
            if max_candidates > 0 and original_pairs_len > max_candidates:
                pairs = pairs[:max_candidates]
                print(
                    f"[{repo}] Processing {len(pairs)} (candidate, modifier) pairs (limited from {original_pairs_len})."
                )
            else:
                print(
                    f"[{repo}] Processing {len(pairs)} (candidate, modifier) pairs in randomized order."
                )

            # Process in randomized order
            for candidate, pm in tqdm(pairs):
                local_total += _process_candidate(candidate, pm, log_dir, repo)
                if check_timeout():
                    return local_total
        else:
            # Sequential processing (original behavior)
            for ext, pm_list in MAP_EXT_TO_MODIFIERS.items():
                for pm in pm_list:
                    candidates = [
                        x
                        for x in entities
                        if Path(x.file_path).suffix == ext and pm.can_change(x)
                    ]
                    if not candidates:
                        continue
                    print(f"[{repo}] Found {len(candidates)} candidates for {pm.name}.")

                    if max_bugs > 0 and len(candidates) > max_bugs:
                        candidates = random.sample(candidates, max_bugs)

                    # Apply max_candidates limit across all processed pairs
                    processed = 0
                    for candidate in tqdm(candidates):
                        if max_candidates > 0 and processed >= max_candidates:
                            return local_total
                        local_total += _process_candidate(candidate, pm, log_dir, repo)
                        processed += 1
                        if check_timeout():
                            return local_total
        return local_total

    total = process_with_timeout()

    shutil.rmtree(repo)
    print(f"Generated {total} bugs for {repo}.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generate bugs for a given repository and commit."
    )
    parser.add_argument(
        "repo",
        type=str,
        help="Name of a SWE-smith repository to generate bugs for.",
    )
    parser.add_argument(
        "-s",
        "--seed",
        type=int,
        default=24,
        help="Seed for random number generator.",
    )
    parser.add_argument(
        "--max_bugs",
        type=int,
        default=-1,
        help="Maximum number of bugs to generate.",
    )
    parser.add_argument(
        "-i",
        "--interleave",
        action="store_true",
        help="Randomize and interleave modifiers instead of processing sequentially.",
    )
    parser.add_argument(
        "--max_entities",
        type=int,
        default=-1,
        help="Maximum number of entities to sample from the repository. Set to -1 to disable sampling.",
    )
    parser.add_argument(
        "--max_candidates",
        type=int,
        default=-1,
        help="Maximum number of (candidate, modifier) pairs to process. Set to -1 to process all.",
    )
    parser.add_argument(
        "-t",
        "--timeout_seconds",
        type=int,
        default=None,
        help="Maximum number of seconds to run generation. Set to None to disable timeout.",
    )

    args = parser.parse_args()
    main(**vars(args))



from swesmith.constants import TODO_REWRITE, CodeEntity
from tree_sitter import Language, Parser, Query, QueryCursor
import tree_sitter_ruby as tsr
import warnings
from swesmith.bug_gen.adapters.utils import build_entity

RUBY_LANGUAGE = Language(tsr.language())


class RubyEntity(CodeEntity):
    @property
    def name(self) -> str:
        query = Query(
            RUBY_LANGUAGE,
            """
            (method name: (identifier) @method.name)
            (singleton_method name: (identifier) @method.name)
            """,
        )
        captures = QueryCursor(query).captures(self.node)
        if "method.name" in captures:
            name_nodes = captures["method.name"]
            if name_nodes:
                return name_nodes[0].text.decode("utf-8")
        return ""

    @property
    def signature(self) -> str:
        query = Query(
            RUBY_LANGUAGE,
            """
            (method body: (body_statement) @method.body)
            (singleton_method body: (body_statement) @method.body)
            """,
        )

        captures = QueryCursor(query).captures(self.node)
        if "method.body" in captures:
            body_nodes = captures["method.body"]
            if not body_nodes:
                return ""
            body = body_nodes[0]
            method_start_row, method_start_col = self.node.start_point
            body_start_row, body_start_col = body.start_point

            src_lines = self.src_code.split("\n")
            if body_start_row == method_start_row:
                line = src_lines[0]
                signature = line[: body_start_col - method_start_col].strip()
                if signature.endswith(";"):
                    signature = signature[:-1].strip()
                return signature
            else:
                signature_lines = src_lines[: body_start_row - method_start_row]
                return "\n".join(signature_lines).strip()
        return ""

    @property
    def stub(self) -> str:
        return f"{self.signature}\n\t# {TODO_REWRITE}\nend"

    @property
    def complexity(self) -> int:
        def walk(node) -> int:
            score = 0

            if node.type in [
                # binary expressions, operators including and, or, ||, &&...
                "binary",
                # blocks
                "block",
                "do_block",
                "block_argument",
                # assignment operators +=, -=, ||=, |=, &&=...
                "operator_assignment",
                # expression modifiers "perform_foo if bar?"
                "if_modifier",
                "rescue_modifier",
                "unless_modifier",
                "until_modifier",
                "while_modifier",
            ]:
                score += 1

            # ternary
            if node.type == "conditional":
                score += 2

            if (
                node.type
                in ["if", "elsif", "else", "ensure", "rescue", "unless", "when"]
                and node.child_count > 0
            ):
                score += 1

            for child in node.children:
                score += walk(child)

            return score

        return 1 + walk(self.node)


def get_entities_from_file_rb(
    entities: list[RubyEntity],
    file_path: str,
    max_entities: int = -1,
) -> None:
    """
    Parse a .rb file and return up to max_entities top-level funcs and types.
    If max_entities < 0, collects them all.
    """
    parser = Parser(RUBY_LANGUAGE)

    file_content = open(file_path, "r", encoding="utf8").read()
    tree = parser.parse(bytes(file_content, "utf8"))
    root = tree.root_node
    lines = file_content.splitlines()

    def walk(node) -> None:
        # stop if we've hit the limit
        if 0 <= max_entities == len(entities):
            return

        if node.type == "ERROR":
            warnings.warn(f"Error encountered parsing {file_path}")
            return

        # ignoring setter and alias methods
        if node.type in [
            "method",
            "singleton_method",
        ]:
            if any(child.type == "body_statement" for child in node.children):
                entities.append(build_entity(node, lines, file_path, RubyEntity))
                if 0 <= max_entities == len(entities):
                    return

        for child in node.children:
            walk(child)

    walk(root)



"""
Base class for JavaScript procedural modifications.
"""

from abc import ABC
from swesmith.bug_gen.procedural.base import ProceduralModifier


class JavaScriptProceduralModifier(ProceduralModifier, ABC):
    """Base class for JavaScript-specific procedural modifications using tree-sitter AST."""

    pass



"""
Profiles module for SWE-smith.

This module contains repository profiles for different programming languages
and provides a global registry for accessing all profiles.
"""

from .base import RepoProfile, registry

# Auto-import all profile modules to populate the registry
from . import c
from . import cpp
from . import csharp
from . import java
from . import javascript
from . import php
from . import python
from . import golang
from . import rust

__all__ = ["RepoProfile", "registry"]



"""
Purpose: Given a run_evaluation log, repair task instances with flaky tests. Specifically, given a log
generated with:

python swesmith/harness/eval.py -d logs/task_insts/*.json -p gold --run_id <run_id>

This script will take the `logs/run_evaluation/<run_id>` artifact and, for each task instance,
make edits to the following:
* `logs/run_validation/<repo>/<instance_id>`
* `logs/task_insts/*.json`

Specifically, the logic for each instance that will be carried out is as follows:

Tests that are found under
* report["tests_status"]["FAIL_TO_PASS"]["failure"]
* report["tests_status"]["PASS_TO_PASS"]["failure"]
will be remove from the "FAIL_TO_PASS" and "PASS_TO_PASS" fields correspond to the `instance_id` assets
under the aforementioned folders.

If the removal of these tests leads to a instance having an empty FAIL_TO_PASS field, then the instance should
be deleted all together. This implies:
* `rm -r logs/run_validation/<repo>/<instance_id>`
* removal of item from `logs/task_insts/*.json`
"""

import argparse
import json
import subprocess
from pathlib import Path
from swebench.harness.constants import (
    FAIL_TO_PASS,
    PASS_TO_PASS,
    KEY_INSTANCE_ID,
    LOG_REPORT,
)
from swesmith.constants import KEY_TIMED_OUT, LOG_DIR_RUN_VALIDATION, LOG_DIR_TASKS
from swesmith.profiles import registry
from tqdm.auto import tqdm


def _remove_task_instance(
    repo: str,
    inst_id: str,
    validation_path: Path,
    task_insts_file,
    task_insts_cache: dict,
) -> bool:
    """
    Remove a task instance from both logs/run_validation and logs/task_insts.
    This is a helper function to avoid code duplication.
    """
    removed_valid, removed_task, removed_branch = False, False, False
    # 1. Remove from logs/run_validation
    if validation_path.exists():
        subprocess.run(["rm", "-rf", str(validation_path)], check=True)
        removed_valid = True
    # 2. Remove from logs/task_insts
    if repo not in task_insts_cache:
        with open(task_insts_file, "r") as f:
            task_insts_cache[repo] = {x[KEY_INSTANCE_ID]: x for x in json.load(f)}
    if inst_id in task_insts_cache[repo]:
        del task_insts_cache[repo][inst_id]
        removed_task = True
    # 3. Optionally, delete branch from remote if exists
    rp = registry.get(repo)
    if inst_id in rp.branches:
        try:
            subprocess.run(
                f"git push --delete origin {inst_id}",
                cwd=rp.repo_name,
                shell=True,
                check=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            removed_branch = True
        except subprocess.CalledProcessError as e:
            print(f"⚠️ Warning: Failed to delete branch {inst_id} from remote: {e}")
    return removed_valid, removed_task, removed_branch


def main(
    eval_logs: list[str],
    logs_validation: Path,
    logs_task_insts: Path,
    dry_run: bool = False,
):
    task_insts_cache = {}
    total_deleted_timeout, total_deleted, total_modified, total_kept = 0, 0, 0, 0

    if dry_run:
        print(
            "🔍 DRY RUN MODE: No changes will be made, only showing what would happen"
        )
    cloned_repos = set()
    for eval_log in eval_logs:
        with open(Path(eval_log) / LOG_REPORT, "r") as f:
            eval_report = json.load(f)
        unresolved = eval_report["ids_unresolved"]
        print(f"Found {len(unresolved)} unresolved instances in {eval_log}")

        if not dry_run:
            print(
                f"⚠️ This will modify logs under {logs_validation} and {logs_task_insts}"
            )
            if input("Continue? [y/N] ") != "y":
                print("Aborting.")
                return

        for inst_id in tqdm(unresolved):
            repo = inst_id.rsplit(".", 1)[0]
            validation_path = logs_validation / repo / inst_id
            task_insts_file = logs_task_insts / f"{repo}.json"
            rp = registry.get(repo)
            _, cloned = rp.clone()
            if cloned:
                cloned_repos.add(repo)

            with open(Path(eval_log) / inst_id / LOG_REPORT, "r") as f:
                report = json.load(f)

            if report.get(KEY_TIMED_OUT, False):
                if not dry_run:
                    removed_valid, removed_task, removed_branch = _remove_task_instance(
                        repo,
                        inst_id,
                        validation_path,
                        task_insts_file,
                        task_insts_cache,
                    )
                    if removed_valid or removed_task or removed_branch:
                        total_deleted_timeout += 1
                else:
                    # In dry run, count all timeout instances as would-be-deleted
                    total_deleted_timeout += 1
                continue

            f2p_fails = report["tests_status"][FAIL_TO_PASS]["failure"]
            p2p_fails = report["tests_status"][PASS_TO_PASS]["failure"]
            f2p_passes = report["tests_status"][FAIL_TO_PASS]["success"]

            if len(f2p_fails) == 0 and len(p2p_fails) == 0:
                # Nothing to do for this instance
                total_kept += 1
                continue

            if len(f2p_passes) == 0:
                # This instance is irreparably broken, remove it
                if not dry_run:
                    removed_valid, removed_task, removed_branch = _remove_task_instance(
                        repo,
                        inst_id,
                        validation_path,
                        task_insts_file,
                        task_insts_cache,
                    )
                    if removed_valid or removed_task or removed_branch:
                        total_deleted += 1
                else:
                    # In dry run, count as would-be-deleted and load cache for simulation
                    total_deleted += 1
                    if repo not in task_insts_cache:
                        try:
                            with open(task_insts_file, "r") as f:
                                task_insts_cache[repo] = {
                                    x[KEY_INSTANCE_ID]: x for x in json.load(f)
                                }
                        except FileNotFoundError:
                            print(
                                f"⚠️ Warning: Task insts file not found: {task_insts_file}"
                            )
                            task_insts_cache[repo] = {}
                continue

            # Keep instance, but remove the failing tests
            # 1. Update logs/run_validation
            if not validation_path.exists():
                print(
                    f"⚠️ Warning: Validation path does not exist for {inst_id}: {validation_path}"
                )
                total_kept += 1
                continue

            with open(validation_path / LOG_REPORT, "r") as f:
                val_report = json.load(f)
            new_f2p = [x for x in val_report[FAIL_TO_PASS] if x not in f2p_fails]
            assert len(new_f2p) > 0, (
                f"Instance {inst_id} should have some FAIL_TO_PASS tests after removing failures"
            )
            new_p2p = [x for x in val_report[PASS_TO_PASS] if x not in p2p_fails]

            if not dry_run:
                if (
                    val_report[FAIL_TO_PASS] == new_f2p
                    and val_report[PASS_TO_PASS] == new_p2p
                ):
                    # No changes needed
                    total_kept += 1
                    continue
                val_report[FAIL_TO_PASS] = new_f2p
                val_report[PASS_TO_PASS] = new_p2p
                with open(validation_path / LOG_REPORT, "w") as f:
                    json.dump(val_report, f, indent=2)
                # 2. Update logs/task_insts
                if repo not in task_insts_cache:
                    with open(task_insts_file, "r") as f:
                        task_insts_cache[repo] = {
                            x[KEY_INSTANCE_ID]: x for x in json.load(f)
                        }
                if inst_id in task_insts_cache[repo]:
                    inst = task_insts_cache[repo][inst_id]
                    inst[FAIL_TO_PASS] = new_f2p
                    inst[PASS_TO_PASS] = new_p2p
                    task_insts_cache[repo][inst_id] = inst
                total_modified += 1
            else:
                # Dry run: Load cache for simulation and count as modified
                if repo not in task_insts_cache:
                    try:
                        with open(task_insts_file, "r") as f:
                            task_insts_cache[repo] = {
                                x[KEY_INSTANCE_ID]: x for x in json.load(f)
                            }
                    except FileNotFoundError:
                        print(
                            f"⚠️ Warning: Task insts file not found: {task_insts_file}"
                        )
                        task_insts_cache[repo] = {}
                total_modified += 1

    # Write all updated task instances back to files
    if not dry_run:
        for repo, insts_dict in task_insts_cache.items():
            task_insts_file = logs_task_insts / f"{repo}.json"
            with open(task_insts_file, "w") as f:
                json.dump(list(insts_dict.values()), f, indent=2)
    else:
        print(f"\n[DRY RUN] Would write updates to {len(task_insts_cache)} repo files:")
        for repo in task_insts_cache.keys():
            task_insts_file = logs_task_insts / f"{repo}.json"
            print(f"[DRY RUN] - {task_insts_file}")

    print("\n=== SUMMARY ===")
    if dry_run:
        print("🔍 DRY RUN MODE - No actual changes were made")
    print(
        f"Total instances processed: {total_deleted_timeout + total_deleted + total_modified + total_kept}\n"
        f"- {'Would delete (Timeout)' if dry_run else 'Deleted'} (timeout): {total_deleted_timeout}\n"
        f"- {'Would delete (No F2P)' if dry_run else 'Deleted'} (no F2P passes): {total_deleted}\n"
        f"- {'Would modify' if dry_run else 'Modified'}: {total_modified}\n"
        f"- Kept unchanged: {total_kept}"
    )

    # Cleanup cloned repos
    for repo in cloned_repos:
        subprocess.run(["rm", "-rf", repo], check=True)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Repair task instances with flaky tests using run_evaluation logs."
    )
    parser.add_argument(
        "eval_logs",
        type=str,
        nargs="+",
        help="Path(s) to run_evaluation log files (JSON).",
    )
    parser.add_argument("--logs_validation", type=Path, default=LOG_DIR_RUN_VALIDATION)
    parser.add_argument("--logs_task_insts", type=Path, default=LOG_DIR_TASKS)
    parser.add_argument(
        "-d",
        "--dry-run",
        action="store_true",
        help="Show what would be done without making any changes",
    )
    args = parser.parse_args()
    # Convert dry-run to dry_run for function call
    main(
        eval_logs=args.eval_logs,
        logs_validation=args.logs_validation,
        logs_task_insts=args.logs_task_insts,
        dry_run=args.dry_run,
    )



import re
import tree_sitter_rust as tsrs
import warnings

from swesmith.constants import TODO_REWRITE, CodeEntity, CodeProperty
from tree_sitter import Language, Parser, Query, QueryCursor
from swesmith.bug_gen.adapters.utils import build_entity

RUST_LANGUAGE = Language(tsrs.language())


class RustEntity(CodeEntity):
    def _analyze_properties(self):
        """Analyze Rust code properties."""
        node = self.node

        if node.type == "function_item":
            self._tags.add(CodeProperty.IS_FUNCTION)

        self._walk_for_properties(node)

    def _walk_for_properties(self, n):
        """Walk the AST and analyze properties."""
        self._check_control_flow(n)
        self._check_operations(n)
        self._check_expressions(n)

        for child in n.children:
            self._walk_for_properties(child)

    def _check_control_flow(self, n):
        """Check for control flow patterns."""
        if n.type in ["for_expression", "while_expression", "loop_expression"]:
            self._tags.add(CodeProperty.HAS_LOOP)
        if n.type == "if_expression":
            self._tags.add(CodeProperty.HAS_IF)
            for child in n.children:
                if child.type == "else_clause":
                    self._tags.add(CodeProperty.HAS_IF_ELSE)
                    break
        if n.type == "match_expression":
            self._tags.add(CodeProperty.HAS_SWITCH)

    def _check_operations(self, n):
        """Check for various operations."""
        if n.type == "index_expression":
            self._tags.add(CodeProperty.HAS_LIST_INDEXING)
        if n.type == "call_expression":
            self._tags.add(CodeProperty.HAS_FUNCTION_CALL)
        if n.type == "return_expression":
            self._tags.add(CodeProperty.HAS_RETURN)
        if n.type in ["let_declaration", "const_item", "static_item"]:
            self._tags.add(CodeProperty.HAS_ASSIGNMENT)

    def _check_expressions(self, n):
        """Check for expression patterns."""
        if n.type == "binary_expression":
            self._tags.add(CodeProperty.HAS_BINARY_OP)
        if n.type == "unary_expression":
            self._tags.add(CodeProperty.HAS_UNARY_OP)
        if n.type == "closure_expression":
            self._tags.add(CodeProperty.HAS_LAMBDA)

    @property
    def complexity(self) -> int:
        """Calculate cyclomatic complexity for Rust code."""

        def walk(node):
            score = 0
            if node.type in [
                "!=",
                "&&",
                "<",
                "<=",
                "==",
                ">",
                ">=",
                "||",
                "match_arm",
                "else_clause",
                "for_expression",
                "while_expression",
                "loop_expression",
                "if_expression",
            ]:
                score += 1

            for child in node.children:
                score += walk(child)

            return score

        return 1 + walk(self.node)

    @property
    def name(self) -> str:
        func_query = Query(RUST_LANGUAGE, "(function_item name: (identifier) @name)")
        func_name = self._extract_text_from_first_match(func_query, self.node, "name")
        if func_name:
            return func_name
        return ""

    @property
    def signature(self) -> str:
        body_query = Query(RUST_LANGUAGE, "(function_item body: (block) @body)")
        matches = QueryCursor(body_query).matches(self.node)
        if matches:
            body_node = matches[0][1]["body"][0]
            body_start_byte = body_node.start_byte - self.node.start_byte
            signature = self.node.text[:body_start_byte].strip().decode("utf-8")
            signature = re.sub(r"\(\s+", "(", signature).strip()
            signature = re.sub(r",\s+\)", ")", signature).strip()
            signature = re.sub(r"\s+", " ", signature).strip()
            return signature
        return ""

    @property
    def stub(self) -> str:
        return f"{self.signature} {{\n    // {TODO_REWRITE}\n}}"

    @staticmethod
    def _extract_text_from_first_match(query, node, capture_name: str) -> str | None:
        """Extract text from tree-sitter query matches with None fallback."""
        matches = QueryCursor(query).matches(node)
        return matches[0][1][capture_name][0].text.decode("utf-8") if matches else None


def get_entities_from_file_rs(
    entities: list[RustEntity],
    file_path: str,
    max_entities: int = -1,
) -> None:
    """
    Parse a .rs file and return up to max_entities top-level funcs and types.
    If max_entities < 0, collects them all.
    """
    parser = Parser(RUST_LANGUAGE)

    file_content = open(file_path, "r", encoding="utf8").read()
    tree = parser.parse(bytes(file_content, "utf8"))
    root = tree.root_node
    lines = file_content.splitlines()

    def walk(node) -> None:
        # stop if we've hit the limit
        if 0 <= max_entities == len(entities):
            return

        if node.type == "ERROR":
            warnings.warn(f"Error encountered parsing {file_path}")
            return

        if node.type == "function_item":
            if _has_test_attribute(node):
                return

            entities.append(build_entity(node, lines, file_path, RustEntity))
            if 0 <= max_entities == len(entities):
                return

        for child in node.children:
            walk(child)

    walk(root)


def _has_test_attribute(node) -> bool:
    possible_att = node.prev_named_sibling
    while possible_att and possible_att.type == "attribute_item":
        if possible_att.text == b"#[test]":
            return True
        possible_att = possible_att.prev_named_sibling
    return False



from dataclasses import dataclass, field
from swebench.harness.constants import TestStatus
from swesmith.constants import ENV_NAME
from swesmith.profiles.base import RepoProfile, registry


@dataclass
class CSharpProfile(RepoProfile):
    """
    Profile for CSharp repositories.
    """

    exts: list[str] = field(default_factory=lambda: [".cs"])


@dataclass
class VirtualClient0bb16489(CSharpProfile):
    owner: str = "microsoft"
    repo: str = "VirtualClient"
    commit: str = "0bb16489e29d2b8ae18b1187ade52cda4eae68bd"
    test_cmd: str = "./build-test.sh"

    @property
    def dockerfile(self):
        return f"""FROM mcr.microsoft.com/devcontainers/dotnet:dev-9.0-noble
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN chmod +x *.sh \
 && ./build.sh \
 && (./build-test.sh || true)
CMD ["/bin/bash"]
"""

    def _is_test_path(self, root: str, file: str) -> bool:
        return (
            file.endswith("Tests.cs")
            or root.endswith(".UnitTests")
            or root.endswith(".FunctionalTests")
        )

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        for line in log.split("\n"):
            line = line.strip()
            for prefix, status in [
                ("Passed", TestStatus.PASSED.value),
                ("Failed", TestStatus.FAILED.value),
                ("Skipped", TestStatus.SKIPPED.value),
            ]:
                if line.startswith(prefix):
                    test_name = line.split()[1]
                    test_status_map[test_name] = status
                    break
        return test_status_map


# Register all CSharp profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, CSharpProfile)
        and obj.__name__ != "CSharpProfile"
    ):
        registry.register_profile(obj)



"""
Purpose: Given a bug patch, generate a GitHub-style issue that describes the bug.

python swesmith/issue_gen/generate.py \
    --dataset logs/experiments/*.json \
    --config configs/issue_gen/*.yaml \
    --model anthropic/claude-3-7-sonnet-20250219 \
    --workers 2 \
    --redo_existing  # Optional: regenerate existing issue texts
"""

import argparse
import jinja2
import json
import litellm
import logging
import os
import random
import shutil
import yaml

from concurrent.futures import ThreadPoolExecutor, as_completed
from datasets import load_dataset
from dotenv import load_dotenv
from litellm import completion, completion_cost
from litellm.utils import get_token_count
from pathlib import Path
from tqdm import tqdm
from tqdm.contrib.logging import logging_redirect_tqdm
from swebench.harness.constants import (
    FAIL_TO_PASS,
    KEY_INSTANCE_ID,
    LOG_TEST_OUTPUT,
)
from swesmith.constants import (
    KEY_PATCH,
    HF_DATASET,
    LOG_DIR_ISSUE_GEN,
    LOG_DIR_RUN_VALIDATION,
    TEST_OUTPUT_END,
    TEST_OUTPUT_START,
)
from swesmith.harness.utils import (
    matches_instance_filter,
    run_patch_in_container,
)
from swesmith.issue_gen.utils import get_test_function
from swesmith.profiles import registry

logging.getLogger("LiteLLM").setLevel(logging.WARNING)
litellm.drop_params = True
litellm.suppress_debug_info = True


TEST_SRC_CODE_PROMPT = r"""
**Test Source Code:**
Use the following test source code to help you write reasonable, effective reproduction code.

{test_src_code}
"""

load_dotenv()

logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s %(message)s")
logger = logging.getLogger(__name__)


def maybe_shorten(text_str: str, max_tokens: int, model: str) -> str:
    """Shorten text if it exceeds the max_tokens limit.
    If shortening, return a string with the first and last max_tokens//2 tokens.
    """
    if get_token_count([{"content": text_str}], model) < max_tokens:
        return text_str
    return text_str[: max_tokens // 2] + "\n\n(...)\n\n" + text_str[-max_tokens // 2 :]


class IssueGen:
    def __init__(
        self,
        config_file: Path,
        workers: int,
        instance_ids: list | None = None,
        dataset_path: str = HF_DATASET,
        redo_existing: bool = False,
    ):
        self.redo_existing = redo_existing
        self.workers = workers

        self.config = yaml.safe_load(config_file.read_text())
        self.model = self.config.get("model", "openai/gpt-4o")
        settings = self.config.get("settings", {})
        self.n_instructions = settings.get("n_instructions", 1)
        self.max_var_tokens = settings.get("max_var_tokens", 10_000)

        data_smith = [x for x in load_dataset(HF_DATASET, split="train")]
        self.dataset = (
            data_smith
            if dataset_path == HF_DATASET
            else json.loads(Path(dataset_path).read_text())
        )
        logger.info(f"Loaded {len(self.dataset)} instances from {dataset_path}")

        # Filter out instances that already have problem statements in HF dataset
        existing_problems = {
            d["instance_id"] for d in data_smith if d.get("problem_statement")
        }
        self.dataset = [
            x for x in self.dataset if x[KEY_INSTANCE_ID] not in existing_problems
        ]
        logger.info(
            f"Found {len(self.dataset)} instances without existing problem statements"
        )

        # Further filter based on other criteria
        self.dataset = sorted(
            [
                x
                for x in self.dataset
                if self._should_do_instance(x, instance_ids, redo_existing, self.model)
            ],
            key=lambda x: x[KEY_INSTANCE_ID],
        )
        logger.info(f"Will create issues for {len(self.dataset)} instances")

        if len(self.dataset) == 0:
            logger.warning(
                "No instances to process after filtering. Exiting gracefully."
            )
            return

        if FAIL_TO_PASS not in self.dataset[0]:
            raise ValueError(
                "Must be called with the result of swesmith.harness.gather, not the _all_patches.json file"
            )
        self.swebv = load_dataset("princeton-nlp/SWE-bench_Verified", split="test")

    def _should_do_instance(
        self, instance: dict, instance_ids: list | None, redo_existing: bool, model: str
    ) -> bool:
        repo = instance["repo"].split("/")[-1]
        output_file = LOG_DIR_ISSUE_GEN / repo / f"{instance[KEY_INSTANCE_ID]}.json"
        if not matches_instance_filter(instance[KEY_INSTANCE_ID], instance_ids):
            return False
        if redo_existing:
            return True
        if not output_file.exists():
            return True
        metadata = json.loads(output_file.read_text())
        if "responses" not in metadata:
            return True
        if model not in metadata["responses"]:
            return True
        return False

    def get_test_output(self, instance: dict) -> str:
        rp = registry.get_from_inst(instance)

        # Get execution output from running pytest for this instance (from validation step)
        test_output_path = (
            LOG_DIR_RUN_VALIDATION
            / instance["repo"].split("/")[-1]
            / instance[KEY_INSTANCE_ID]
            / LOG_TEST_OUTPUT
        )
        if not test_output_path.exists():
            run_patch_in_container(
                instance,
                instance["repo"].split("/")[-1],
                LOG_DIR_RUN_VALIDATION,
                rp.timeout,
                patch=instance[KEY_PATCH],
            )
        test_output = test_output_path.read_text()

        return maybe_shorten(
            test_output[
                test_output.find(TEST_OUTPUT_START)
                + len(TEST_OUTPUT_START) : test_output.find(TEST_OUTPUT_END)
            ],
            self.max_var_tokens,
            self.model,
        )

    def get_test_functions(self, instance: dict) -> tuple[list[str], list[str]]:
        """
        Get the source code for tests associated with the instance.

        Returns:
            list of test functions, list of repos to remove
        """
        test_funcs = []
        repos_to_remove = []
        test_idxs = list(range(len(instance[FAIL_TO_PASS])))
        random.shuffle(test_idxs)
        for test_idx in test_idxs:
            test_func = get_test_function(instance, test_idx)
            if test_func["cloned"]:
                repos_to_remove.append(test_func["repo_name"])
            test_funcs.append(test_func["test_src"])
        return test_funcs, repos_to_remove

    def get_demo_issues(self) -> list[str]:
        """
        Get a list of demonstration issues from the config file.
        """
        problem_statements = [
            maybe_shorten(instance["problem_statement"], 2000, self.model)
            for instance in self.swebv
        ]  # type: ignore[index]
        random.shuffle(problem_statements)
        return problem_statements

    def generate_issue(self, instance: dict) -> dict:
        # Set up logging information
        repo = instance["repo"].split("/")[-1]
        inst_dir = LOG_DIR_ISSUE_GEN / repo
        inst_dir.mkdir(parents=True, exist_ok=True)

        output_file = inst_dir / f"{instance[KEY_INSTANCE_ID]}.json"
        output_file_exists = output_file.exists()

        # Get a reference instance from SWE-bench
        instance_curr = instance.copy()

        def format_prompt(prompt: str | None, config: dict, candidate: dict) -> str:
            if not prompt:
                return ""
            env = jinja2.Environment()

            def jinja_shuffle(seq):
                result = list(seq)
                random.shuffle(result)
                return result

            env.filters["shuffle"] = jinja_shuffle
            template = env.from_string(prompt)
            return template.render(**candidate, **config.get("parameters", {}))

        metadata = {}
        if output_file_exists:
            metadata = json.loads(output_file.read_text())

        if "messages" not in metadata:
            # Generate prompt
            messages = [
                {"content": self.config["system"], "role": "system"},
            ]
            if self.config["demonstration"]:
                messages.append(
                    {
                        "content": format_prompt(
                            self.config["demonstration"],
                            self.config,
                            {"demo_problem_statements": self.get_demo_issues()},
                        ),
                        "role": "user",
                    },
                )
            test_funcs, repos_to_remove = self.get_test_functions(instance_curr)
            messages.append(
                {
                    "content": format_prompt(
                        self.config["instance"],
                        self.config,
                        instance_curr
                        | {
                            "test_output": self.get_test_output(instance_curr),
                            "test_funcs": test_funcs,
                        },
                    ),
                    "role": "user",
                },
            )
            metadata = {"messages": messages, "repos_to_remove": repos_to_remove}
            with open(output_file, "w") as f_:
                json.dump(metadata, f_, indent=4)
        else:
            # If messages already exist, get repos_to_remove from existing metadata
            _, repos_to_remove = self.get_test_functions(instance_curr)

        # Generate n_instructions completions containing problem statements
        response = completion(
            model=self.model, messages=messages, n=self.n_instructions, temperature=0
        )

        cost = completion_cost(response)
        metadata["cost"] = (0 if "cost" not in metadata else metadata["cost"]) + cost

        # Extract problem statements from response
        problem_statements = [
            choice.message.content  # type: ignore[attr-defined]
            for choice in response.choices  # type: ignore[attr-defined]
        ]

        if "responses" not in metadata:
            # Initialize responses dict if it doesn't exist
            metadata["responses"] = {}
        elif self.model in metadata["responses"]:
            # If responses for this model already exist, prepend them to the new ones
            problem_statements = metadata["responses"][self.model] + problem_statements

        # Add/update the response for current model
        metadata["responses"][self.model] = problem_statements

        with open(output_file, "w") as f_:
            json.dump(metadata, f_, indent=4)

        return {
            "status": "completed",
            "cost": cost,
            "repos_to_remove": repos_to_remove,
        }

    def _cleanup_repos(self, repos_to_remove):
        """Remove cloned repositories."""
        if not repos_to_remove:
            return

        logger.info(f"Cleaning up {len(repos_to_remove)} cloned repositories...")
        for repo_path in repos_to_remove:
            if os.path.exists(repo_path):
                try:
                    shutil.rmtree(repo_path)
                    logger.debug(f"Removed repository: {repo_path}")
                except Exception as e:
                    logger.warning(f"Failed to remove repository {repo_path}: {e}")
        logger.info("Repository cleanup completed.")

    def run(self):
        # Check if dataset is empty (initialization returned early)
        if not hasattr(self, "dataset") or len(self.dataset) == 0:
            logger.info("No instances to process. Exiting.")
            return

        stats = {
            "💰": 0.0,
            "⏭️": 0,
            "❌": 0,
            "✅": 0,
        }

        # Track repos to remove for cleanup
        all_repos_to_remove = set()

        # Create a thread pool and call generate_issue for each instance
        with ThreadPoolExecutor(max_workers=self.workers) as executor:
            futures = []
            for instance in self.dataset:
                future = executor.submit(self.generate_issue, instance)
                futures.append(future)

            # Wait for all futures to complete
            with logging_redirect_tqdm():
                with tqdm(total=len(futures), desc="Generating issues") as pbar:
                    for future in as_completed(futures):
                        try:
                            result = future.result()
                        except KeyboardInterrupt:
                            raise
                        except Exception as e:
                            logger.error(
                                f"Error processing instance: {e}", exc_info=True
                            )
                            stats["❌"] += 1
                            continue
                        if result["status"] == "skipped":
                            stats["⏭️"] += 1
                        elif result["status"] == "completed":
                            stats["✅"] += 1
                            stats["💰"] += result["cost"]
                            # Collect repos to remove
                            if "repos_to_remove" in result:
                                all_repos_to_remove.update(result["repos_to_remove"])
                        pbar.set_postfix(stats, refresh=True)
                        pbar.update(1)

        # Cleanup cloned repositories
        self._cleanup_repos(all_repos_to_remove)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "-d",
        "--dataset_path",
        type=str,
        help="Path to the dataset to annotate with bugs.",
        default=HF_DATASET,
    )
    parser.add_argument(
        "-i",
        "--instance_ids",
        type=str,
        help="Instance IDs to evaluate (supports exact matches and glob patterns like 'repo__name.*')",
        nargs="+",
    )
    parser.add_argument(
        "-c", "--config_file", type=Path, help="Path to the template config file."
    )
    parser.add_argument(
        "-w",
        "--workers",
        type=int,
        help="Number of workers to use for generation.",
        default=1,
    )
    parser.add_argument(
        "-r",
        "--redo_existing",
        action="store_true",
        help="Whether to redo instances that already have an output file.",
    )
    args = parser.parse_args()
    if args.workers == 1:
        logger.warning(
            "Using only 1 worker for generation. You can speed up the generation by setting --workers > 1."
        )
    IssueGen(**vars(args)).run()
